#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2010-2011 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           buttons.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import utils
import types
import glob
import pygame
from pygame.constants import *
from base import Widget
from SPConstants import *
from text import Label
from funcs import get_boxes, make_button_bg_dynamic
from SPSpriteUtils import SPSprite, SPGroup

class RadioButton(Widget):
    """Radiobuttons are buttons that has two states, selected or not selected.
    The differences with other 'state' buttons is that these buttons are usually 
    grouped so that only one can be selected at a time."""
    def __init__(self, group, text, pos, fsize=18, padding=8, name='', **kwargs):
        """Radiobutton with a optional text label.
        group - the widget to which this button belongs. (must be None or another radiobutton)
        text - string to display
        pos - position to display the box
        fsize - Font size
        padding - space in pixels around the text
        name - string to indicate this object
        kwargs - bgcol, fgcol
        """
        self.box_pos = pos
        self.image_pos = (pos[0]+1, pos[1]+1)
        self.name = name
        
        if not group:
            self._rb_group = [self]
        else:
            self._rb_group = group._rb_group + [self]
            for rb in self._rb_group:
                rb._rb_group = self._rb_group
        self._rb_selected = False
        box = pygame.Surface((18, 18))
        box.fill(WHITE)
        rect = box.get_rect()
        lbl = Label(text, (pos[0] + 10 + rect.width, pos[1]), fsize=18, transparent=True, \
                            fgcol=WHITE, bold=True)
        lbl.display_sprite()
        pygame.draw.line(box, GREY30, (0, 0), (18, 0), 1)
        pygame.draw.line(box, GREY30, (0, 0), (0, 18), 1)
        pygame.draw.line(box, GREY30, (17, 0), (17, 17), 1)
        pygame.draw.line(box, GREY30, (1, 17), (17, 17), 1)
        self.box = box.convert()
        Widget.__init__(self, box, name=name)
        self.selected_image = utils.load_image(os.path.join(self.THEME['defaultpath'], 'radiobut_selected.png'))
        self.moveto(self.box_pos)
        self.connect_callback(self._cbf_on_select, MOUSEBUTTONUP)
    
    def _rb_select(self):
        print 'selecting:', self.name
        self.image = self.selected_image
        self.moveto(self.image_pos)
        self._rb_selected = True
        self.display_sprite()
    
    def _rb_unselect(self):
        print 'unselecting:', self.name
        self.image = self.box
        self.moveto(self.box_pos)
        self._rb_selected = False
        self.display_sprite()
    
    def _cbf_on_select(self, *args):
        print "_cbf %s: selection = %s" % (self.name, self._rb_selected)
        if self._rb_selected:
            self._rb_unselect()
        else:
            for rb in self._rb_group:
                if not rb is self:
                    rb._rb_unselect()
            self._rb_select()
            
    def get_selected_name(self):
        for rb in self._rb_group:
            if rb._rb_selected:
                return rb.name
        return ''
        
    def get_group(self):
        return self._rb_group

class Button(Widget):
    def __init__(self, txt, pos=(0, 0), fsize=18, padding=4, name='', fbold=True, **kwargs):
        """Button which shows a text
        txt - string to display
        pos - position to display the box
        fsize - Font size
        padding - space in pixels around the text
        name - string to indicate this object
        kwargs - bgcol, fgcol
        """
        Widget.__init__(self)
        if kwargs.has_key('fgcol'):
            fgcol = kwargs['fgcol']
        else:
            fgcol = self.THEME['button_fg_color']
        self.kwargs = kwargs
        self.ImSelected = False
        if type(txt) in (types.StringType, types.UnicodeType):
            s = utils.char2surf(txt, fcol=fgcol, fsize=fsize, ttf=TTF, bold=fbold)
            r = s.get_rect()
            r.w += padding*2
            r.h += padding*2
        else:
            s = txt
            r = s.get_rect()
            r.w += padding*2
            r.h += padding*2
        if kwargs.has_key('maxlength') and r.w > kwargs['maxlength']:
            r.w = kwargs['maxlength']
        self.pos = pos
        self.padding = padding
        self.name = name
        self.r = r
        self._setup(s)
        
    def _setup(self, s):
        if self.kwargs.has_key('bgcol'):
            bgcol = self.kwargs['bgcol']
        else:
            bgcol = self.THEME['button_bg_color']
        self._but, self._but_hover, self._but_selected = get_boxes(\
                                                bgcol,\
                                                self.THEME['button_color_hover'],\
                                                self.r, \
                                                self.THEME['button_selected_color'])

        self._but.blit(s, (self.padding, self.padding))
        self._but_hover.blit(s, (self.padding, self.padding))
        self._but_selected.blit(s, (self.padding, self.padding))
        self.image = self._but
        self.rect  = self.image.get_rect()
        self.rect.move_ip(self.pos)
    
    def mouse_hover_enter(self):
        if self.ImSelected:
            return
        self.hover_active = True
        self.image = self._but_hover
        self.erase_sprite()
        self.display_sprite()
        
    def mouse_hover_leave(self):
        if self.ImSelected:
            return
        self.hover_active = False
        self.image = self._but
        self.erase_sprite()
        self.display_sprite()
    
    def select(self):
        self.ImSelected = True
        self.erase_sprite()
        self.image = self._but_selected
        self.display_sprite()
        
    def unselect(self):
        self.ImSelected = False
        self.erase_sprite()
        self.image = self._but
        self.display_sprite()
        
    def get_button_rect(self):
        return self.rect
    
class ImgButton(Button):
    def __init__(self, path, pos=(0, 0),padding=4, name='', **kwargs):
        """Button which shows an image.
        path - must be the path to the image file or a pygame surface
        pos - position to display the box
        rect - Rect indicating the size of the box
        padding - space in pixels around the text
        name - string to indicate this object
        """
        if type(path) in types.StringTypes:
            image = utils.load_image(path)
        else:
            image = path
        Button.__init__(self, image, pos, None, padding=padding, name=name)

class ImgTextButton(ImgButton):
    def __init__(self, path, text, pos=(0, 0), textpos=2, padding=4, fsize=24,\
                  fgcol=BLACK, name='', **kwargs):
        """Button which shows an image with text.
        path - must be the path to the image file or a pygame surface
        pos - position to display the box
        rect - Rect indicating the size of the box
        padding - space in pixels around the text
        name - string to indicate this object
        text - text to display. String will be split on \n.
        textpos - position of the text, 1 means left from the image, 2 means right from the image
        """
        if type(path) in types.StringTypes:
            image = utils.load_image(path)
        else:
            image = path
        surflist = []
        for line in text.split('\n'):
            if line == '':
                line = ' '
            surflist.append(utils.char2surf(line, fcol=fgcol, fsize=fsize, ttf=TTF))
        w = max([s.get_width() for s in surflist])
        h = surflist[0].get_height() 
        totalh = h * len(surflist)
        textsurf = pygame.Surface((w, totalh), SRCALPHA)
        y = 0
        for s in surflist:
            textsurf.blit(s, (0, y))
            y += h
        w = image.get_width() + w + padding * 2
        h = max(image.get_height(),  totalh)
        surf = pygame.Surface((w, h), SRCALPHA)
        image_y = (surf.get_height() - image.get_height()) / 2
        text_y = (surf.get_height() - textsurf.get_height()) / 2 
        if textpos == 1:
            surf.blit(textsurf, (0, text_y))
            surf.blit(image, (textsurf.get_width() + padding, image_y))
        else:
            surf.blit(image, (0, image_y))
            surf.blit(textsurf, (image.get_width() + padding, text_y))
        ImgButton.__init__(self, surf, pos, padding=padding, name=name)


class TransImgButton(ImgButton):
    def __init__(self, path, hpath, pos=(0, 0), padding=4,text='',fsize=24, fcol=BLACK, name='', **kwargs):
        """Button which shows an image but has a fully transparent background.
        When a hover event occurs the image is changed.
        path - must be the path to the image file or a pygame surface
        hpath - must be the path to the image file or a pygame surface that will
                be shown when a hover event occurs.
        pos - position to display the box
        rect - Rect indicating the size of the box
        padding - space in pixels around the text
        name - string to indicate this object
        text - when a sting is given it will be blitted over the image.
        fsize - fontsize.
        """
        self.text = text
        self.fsize = fsize
        if type(path) in types.StringTypes:
            image = utils.load_image(path)
        else:
            image = path
        if hpath:
            if type(hpath) in types.StringTypes:
                self.hs = utils.load_image(hpath)
            else:
                self.hs = hpath
        else:
            self.hs = utils.grayscale_image(image.convert_alpha())
        if self.text:
            s = utils.char2surf(text, fsize, fcol)
            r = image.get_rect()
            sr = s.get_rect()
            sr.center = r.center
            image.blit(s, sr)
            self.hs.blit(s, sr)
        ImgButton.__init__(self, image, pos, padding=padding, name=name)
        
    def _setup(self, s):
        self.trans = True
        self._but = pygame.Surface(self.r.size,SRCALPHA)
        #self._but.fill((0, 0, 0, 0))
        self._but_hover = pygame.Surface(self.r.size, SRCALPHA)
        
        self._but.blit(s, (self.padding, self.padding))
        self._but_hover.blit(self.hs, (self.padding, self.padding))
    
        self._but_selected = self._but_hover
        self.image = self._but
        self.rect  = self.image.get_rect()
        SPSprite.__init__(self, self.image, name=self.name)
        self.rect.move_ip(self.pos)

class TransImgTextButton(TransImgButton):
    def __init__(self, path, hpath, text, pos=(0, 0), textpos=2, padding=4, fsize=24,\
                  fgcol=BLACK, name='', **kwargs):
        """Button which shows an image and text.
        path - must be the path to the image file.
        hpath - 'rollover' image path.
        pos - position to display the box
        rect - Rect indicating the size of the box
        padding - space in pixels around the text
        name - string to indicate this object
        text - text to display. String will be split on \n.
        textpos - position of the text, 1 means left from the image, 2 means right from the image
        """
        image = utils.load_image(path)
        imageh = utils.load_image(hpath)
        surflist = []
        surfhlist = []
        for line in text.split('\n'):
            if line == '':
                line = ' '
            surflist.append(utils.char2surf(line, fcol=fgcol, fsize=fsize, ttf=TTF))
            surfhlist.append(utils.char2surf(line, fcol=GREY, fsize=fsize, ttf=TTF))
        w = max([s.get_width() for s in surflist])
        h = surflist[0].get_height() 
        totalh = h * len(surflist)
        textsurf = pygame.Surface((w, totalh), SRCALPHA)
        y = 0
        for s in surflist:
            textsurf.blit(s, (0, y))
            y += h
        textsurf_h = pygame.Surface((w, totalh), SRCALPHA)
        y = 0
        for s in surfhlist:
            textsurf_h.blit(s, (0, y))
            y += h
        w = image.get_width() + w + padding * 2
        h = max(image.get_height(),  totalh)
        surf = pygame.Surface((w, h), SRCALPHA)
        hsurf = surf.copy()
        image_y = (surf.get_height() - image.get_height()) / 2
        text_y = (surf.get_height() - textsurf.get_height()) / 2
        if textpos == 1:
            surf.blit(textsurf, (0, text_y))
            surf.blit(image, (textsurf.get_width() + padding, image_y))
            hsurf.blit(textsurf_h, (0, text_y))
            hsurf.blit(imageh, (textsurf_h.get_width() + padding, image_y))
        else:
            surf.blit(image, (0, image_y))
            surf.blit(textsurf, (image.get_width() + padding, text_y))
            hsurf.blit(imageh, (0, image_y))
            hsurf.blit(textsurf_h, (imageh.get_width() + padding, text_y))
        
        TransImgButton.__init__(self, surf, hsurf, pos=pos, padding=padding, \
                                    fsize=fsize, fcol=fgcol, name=name)

class ButtonDynamic(Button):
    def __init__(self, txt, pos=(0, 0), fsize=18, bold=False, padding=8, name='', \
                 length=None, colorname='blue', sizename='54px', fgcol=WHITE, **kwargs):
        """Button which shows a text on a background with round corners.
        txt - string to display
        pos - position to display the box
        fsize - Font size
        padding - space in pixels around the text
        sizename - name for the kind of image set to use.
            supported size names are: '36px','54px','81px'.
        name - string to indicate this object
        kwargs - bgcol, fgcol
        """
        self.colorname = colorname
        self.sizename = sizename
        self.padding = padding
        self.kwargs = kwargs
        Button.__init__(self, txt,pos=pos, fsize=fsize, padding=padding, name=name, fbold=bold, fgcol=fgcol,**kwargs)
        
    def _setup(self, s):
        self._but, self._but_hover = make_button_bg_dynamic(self.r.width + self.padding*2, \
                                                            self.sizename, \
                                                            self.colorname,\
                                                            colorname_ro='black',\
                                                            THEME=self.THEME)
        r = self._but.get_rect()
        size = r.size
        sr = s.get_rect()
        sr.center = r.center
                
        self._but.blit(s, sr)
        self._but_hover.blit(s, sr)
        self.image = self._but
        self.rect  = self.image.get_rect()
        SPSprite.__init__(self, self.image, name=self.name)
        self.rect.move_ip(self.pos)

class ImgButtonDynamic(ButtonDynamic):
    def __init__(self, path, pos=(0, 0),padding=4, name='', \
                 length=None, colorname='blue', sizename='54px', **kwargs):
        """Button which shows an image on a background with round corners.
        path - must be the path to the image file or a pygame surface
        pos - position to display the box
        rect - Rect indicating the size of the box
        padding - space in pixels around the text
        sizename - name for the kind of image set to use.
            supported size names are: '36px','54px','81px'.
        name - string to indicate this object
        """
        if type(path) in types.StringTypes:
            image = utils.load_image(path)
        else:
            image = path
        ButtonDynamic.__init__(self, image, pos, None, padding=padding, name=name, \
                             length=length, colorname=colorname, sizename=sizename)

class SimpleButton(Button):
    """Button which shows a text and returns the value given by 'data'
        when a MOUSEBUTTONUP event occurs.
        This differs from a normal Button which should be connect a 
        function."""
    def __init__(self, txt, pos=(0, 0), fsize=18, padding=4, data=True, **kwargs):
        """
        txt - string to display
        pos - position to display the box
        fsize - Font size
        padding - space in pixels around the text
        data - Optional data to pass to the caller, defaults to True.        
        """
        Button.__init__(self, txt, pos, fsize, padding)
        self.connect_callback(self._cbf, MOUSEBUTTONUP)
        self.data = data
        
    def _cbf(self, *args):
        return (self.data, )
        
class SimpleButtonDynamic(ButtonDynamic):
    """Button which extends ButtonDynamic and returns the value given by 'data'
        when a MOUSEBUTTONUP event occurs.
        This differs from a normal ButtonDynamic which should be connect a 
        function."""
    def __init__(self, txt, pos=(0, 0), fsize=18, padding=4, data=True, \
                 length=None, colorname='blue', sizename='54px', fgcol=WHITE,**kwargs):
        ButtonDynamic.__init__(self, txt, pos, fsize, padding, fgcol=fgcol, \
                             length=length, colorname=colorname, sizename=sizename)
        self.connect_callback(self._cbf, MOUSEBUTTONUP)
        self.data = data
    def _cbf(self, *args):
        return (self.data, )

class SimpleTransImgButton(TransImgButton):
    """Button which extends TransImgButton and returns the value given by 'data'
        when a MOUSEBUTTONUP event occurs.
        This differs from a normal TransImgButton which should be connect a 
        function."""
    def __init__(self, path, hpath, pos=(0, 0), padding=4,text='',fsize=24, fcol=BLACK,\
                  data=True, **kwargs):
        TransImgButton.__init__(self, path, hpath, pos, padding,text,fsize, fcol)
        self.connect_callback(self._cbf, MOUSEBUTTONUP)
        self.data = data
    def _cbf(self, *args):
        return (self.data, )    

class PrevNextButton:
    """Button that can switch between two images in a toggle like fashion.
    The data returnt to the callback function is also changed when the button
    image is changed"""
    def __init__(self, pos, cbf, prevname, nextname, states=['prev','next']):
        self.pos = pos
        self._cbf = cbf
        self.prevname = prevname
        self.nextname = nextname
        self._setup()
        
    def _setup(self):
        pp = self.prevname
        np = self.nextname
        self.state = self.states[0]

        previmg = utils.load_image(pp)
        self.nextimg = utils.load_image(np)
        self.but = ImgButton(previmg, self.pos, name='prevnext')
        self.previmg = self.but._but.convert_alpha()# image is now blitted on a button surface
        self.previmg_hover = self.but._but_hover.convert_alpha()
        self.current = self.previmg
        
        temp = ImgButton(self.nextimg, self.pos, name='prevnext')
        self.nextimg = temp._but.convert_alpha()
        self.nextimg_hover = temp._but_hover.convert_alpha()
        self.rect = self.nextimg.get_rect()
        self.but.connect_callback(self._cbf, MOUSEBUTTONUP, self.states[1])
    
    def get_actives(self):
        return self.but
    
    def toggle(self):
        """When called the button switch images."""
        if self.current is self.previmg:
            self.current = self.nextimg
            self.but._but = self.nextimg
            self.but._but_hover = self.nextimg_hover
            self.state = self.states[1]
            self.but.connect_callback(self._cbf, MOUSEBUTTONUP, self.states[1])
        else:
            self.current = self.previmg
            self.but._but = self.previmg
            self.but._but_hover = self.previmg_hover
            self.state = self.states[0]
            self.but.connect_callback(self._cbf, MOUSEBUTTONUP, self.states[0])
        self.but.mouse_hover_leave()

    def get_state(self):
        return self.state

    # abstract methods so that this widget acts like a regular widget
    def enable(self, enable):
        self.but.enable(enable)
    def display_sprite(self, pos=None):
        self.but.display_sprite(pos)
    def erase_sprite(self):
        self.but.erase_sprite()

class TransPrevNextButton(PrevNextButton):
    """Transparent button that can switch between two images in a toggle like fashion.
    The data returnt to the callback function is also changed when the button
    image is changed"""
    def __init__(self, pos, cbf, prevname, hprevname, nextname, hnextname, states=['prev','next']):
        self._cbf = cbf
        self.pos = pos
        self.prevname = prevname
        self.hprevname = hprevname
        self.nextname = nextname
        self.hnextname = hnextname
        self.states = states
        PrevNextButton.__init__(self, pos, cbf, prevname, nextname)
                
    def _setup(self):
        self.state = self.states[0]
        pp =  self.prevname
        hpp =  self.hprevname
        np = self.nextname
        hnp = self.hnextname
        self.but = TransImgButton(pp, hpp, self.pos, name='prevnext')
        # this is needed for Widet.enable, normal enable/disable doesn't work
        # for per pixel alphas
        s = pygame.Surface(self.but._but.get_size(), SRCALPHA)
        s.fill((0, 0, 0, 80))
        self.but.trans = s
        self.previmg = self.but._but.convert_alpha()# image is now blitted on a button surface
        self.previmg_hover = self.but._but_hover.convert_alpha()
        self.current = self.previmg

        temp = TransImgButton(np, hnp, self.pos, name='prevnext')
        self.nextimg = temp._but.convert_alpha()
        self.nextimg_hover = temp._but_hover.convert_alpha()
        self.rect = self.nextimg.get_rect()
        self.but.connect_callback(self._cbf, MOUSEBUTTONUP, self.states[1])
        

class ChartButton(ImgButton):
    def __init__(self, pos, cbf):
        bname = 'core_chart_button.png'
        p = os.path.join(self.THEMESPATH, self.THEME['self.THEME'], bname)
        if not os.path.exists(p):
            p = os.path.join(DEFAULTTHEMESPATH, bname)
        ImgButton.__init__(self, p, pos, name='Chart')
        self.connect_callback(cbf, MOUSEBUTTONUP, 'Chart')
    
class DiceButtons(Widget):
    def __init__(self, pos, actives, cbf, usebutton=True):
        Widget.__init__(self)
        self.logger = logging.getLogger("childsplay.SPSpriteGui.DiceButtons")
        self.actives = actives
        self.theme = self.THEME['theme']
        self.minimallevel = 1
        self.maxlevels = 6
        self._isenabled = True
        self._setup(self.theme, pos, actives, cbf, usebutton=True)
        self.showme = True
        
    def _setup(self, THEME, pos, actives, cbf, usebutton=True):
        bname = 'dice-1.png'
        p = os.path.join(self.THEMESPATH, self.theme, bname)
        if not os.path.exists(p):
            p = DEFAULTTHEMESPATH
        else:
            p = os.path.join(self.THEMESPATH, self.theme)
        dicelist = glob.glob(os.path.join(p, 'dice*.png'))
        self._butdict = {}
        self._current_but = None
        self.UseDice = usebutton
        count = 1
        try:
            dicelist.sort()
            for diceimg in dicelist:
                but = ImgButton(diceimg, pos, name='Dice')
                but.connect_callback(cbf, MOUSEBUTTONUP, count)
                self._butdict[count] = but
                count += 1
        except StandardError, info:
            self.logger.exception("Can't load dice images for buttons: %s" % info)
            self.logger.error('Disable the dice')
            self._current_but = self._butdict[1]
            self.UseDice = False
    
    def isenabled(self):
        return self._isenabled
    
    def set_minimal_level(self, level):
        self.logger.debug("Minimallevel set to %s" % level)
        self.minimallevel = level
        self.next_level(1)

    def get_minimal_level(self):
        return self.minimallevel

    def next_level(self, level):
        self.logger.debug("level %s called" % level)
        if level == 0:
            self.logger.debug("level called with 0, set it to 1")
            level = 1
#        if not self.UseDice:
#            self.logger.debug("self.theme doesn't use dice")
#            return 
        if self._current_but:
            self._current_but.erase_sprite()
            self.actives.remove(self._current_but)
        level = min(self.maxlevels, level + self.minimallevel - 1)
        self._current_but = self._butdict[level]
        self.actives.add(self._current_but)
        if self._isenabled:
            self.actives.add(self._current_but)
            self._current_but.enable(True)
        self.show()
            
    def enable(self, enable):
        self.logger.debug("enable called with: %s" % enable)
        if not self.UseDice:
            self.logger.debug("self.theme doesn't use dice")
            return
        self._current_but.enable(enable)
        self._isenabled = enable
    
    def erase(self):
        self.logger.debug("erase called")
        if not self.UseDice:
            self.logger.debug("self.theme doesn't use dice")
            return
        self._current_but.erase_sprite()
        self.showme = False
        
    def show(self):
#        if not self._isenabled:
#            return
        self.logger.debug("display called")
        if not self.UseDice:
            self.logger.debug("self.theme doesn't use dice")
            return
        self._current_but.display_sprite()
        self.showme = True

class StarButton(DiceButtons):
    """Special level indicator for the level indicator used by the braintrainer version.
    This uses a special dialog, SPWidgets.StarButtonsDialog.
    It derived from the DiceButtons class so that the changes to the core are minimal."""
    def __init__(self, pos, actives, cbf, lock, usebutton=True, THEME='seniorplay'):
        self.maxlevels = 6
        self.cbf = cbf
        self._current_but = None
        self.UseDice = usebutton
        
        self.minimallevel = 1
        self.lock = lock
        DiceButtons.__init__(self, pos, actives, cbf, usebutton=True)
        self.logger = logging.getLogger("childsplay.SPSpriteGui.StarButton")
    
    def _setup(self, THEME, pos, actives, cbf, usebutton=True):
        self.imgstar0 = utils.load_image(os.path.join(THEMESPATH, self.theme,'star0.png'))
        self.imgstar1 = utils.load_image(os.path.join(THEMESPATH, self.theme,'star1.png'))
        self.rectstar = self.imgstar0.get_rect()
        #self.image = pygame.Surface((self.rectstar.w * self.maxlevels, self.rectstar.h))
        #self.image.fill(self.THEME['starbutton_bg_color'])
        self.image = pygame.Surface((self.rectstar.w * self.maxlevels, self.rectstar.h), SRCALPHA)
        self.image.fill((0, 0, 0, 0))
        self.pos = pos
        self._set_level(1)
        
    def next_level(self, level):
        self.logger.debug("level %s called" % level)
        if level == 0:
            self.logger.debug("level called with 0, set it to 1")
            level = 1
        if not self.UseDice :
            self.logger.debug("THEME doesn't use dice")
            return 
        self._set_level(level)
    
    def _start_dlg(self,sprite, event, data):
        self.lock.acquire()
        sbd = StarButtonsDialog(self.maxlevels, self.minimallevel)
        sbd.set_level(data[0])
        sbd.run()
        result = sbd.get_result()[0]
        if result:
            self._set_level(result- (self.minimallevel - 1))
            self.cbf(sprite, event, result - (self.minimallevel - 1))
        else:
            self.cbf(sprite, event, result)
        self.lock.release()

    def _set_level(self, level):
        self.logger.debug("_set_level called with: %s" % level)
        if self._current_but:
            self.actives.remove(self._current_but)
        x = 0
        level = min(self.maxlevels, level + self.minimallevel-1)
        if level < self.minimallevel:
            level = self.minimallevel
        for i in range(0, level):
            self.image.blit(self.imgstar1, (x, 0))
            x += self.rectstar.w
        for i in range(level, self.maxlevels):
            self.image.blit(self.imgstar0, (x, 0))
            x += self.rectstar.w
        self._current_but = TransImgButton(self.image, None, self.pos, name='Star')
        if self._isenabled:
            self._current_but.connect_callback(self._start_dlg, MOUSEBUTTONUP, level)
            self.actives.add(self._current_but)
        self.show()
        
class StarButtonsDialog(Widget):
    """Special dialog with a level indicator used by the braintrainer version.
    This is intended to be controlled by the SPSpriteGui.StarButtons object"""
    def __init__(self, maxlevels, start_level):
        Widget.__init__(self)
        self.theme = self.THEME['theme']
        self.runloop = True
        self.maxlevels = maxlevels
        self.start_level = start_level
        self.starlist =[]
        self.imgstar0 = utils.load_image(os.path.join(THEMESPATH, self.theme,'star0_b.png'))
        self.imgstar1 = utils.load_image(os.path.join(THEMESPATH, self.theme,'star1_b.png'))
        self.actives = SPGroup(self.screen, self.backgr)
        self.actives.set_onematch(True)
        self.scrclip = self.screen.get_clip()
        self.screen.set_clip((0, 0, 800, 600))    
        self.dim = utils.Dimmer()
        self.dim.dim(darken_factor=self.THEME['starbuttonsdialog_darkenfactor'], color_filter=self.THEME['starbuttonsdialog_dim_color'])
        x = 40
        y = 200
        for i in range(self.maxlevels):
            b = SPSprite(self.imgstar0)
            b.moveto((x, y), hide=True)
            b.connect_callback(self.cbf, MOUSEBUTTONUP, i+1)
            self.starlist.append(b)
            x += self.imgstar0.get_width()
        # set first star to active
        self.starlist[0].image = self.imgstar1
        self.actives.add(self.starlist)
        
        self.buttons_list = []
        x = 100
        y = 400
        
        for buttxt in [_("Cancel"), _("OK")]:
            b = Button(buttxt, (x, y), fsize=self.THEME['starbuttonsdialog_fsize'], name=buttxt)
            b.connect_callback(self.but_cbf, MOUSEBUTTONUP, buttxt)
            self.actives.add(b)
            self.buttons_list.append(b)# keep the users order for showing (see below)
            x += 300
        
    def cbf(self, widget, event, data):
        self.result = data 
        self.set_level(data[0])
            
    def but_cbf(self, widget, event, data):
        if data[0] == _("Cancel"):
            self.result = [None]
        self.runloop = False

    def get_result(self):
        if not self.result:
            return [None]
        return self.result
        
    def run(self):
        self.result = None
        for b in self.actives:
            b.display_sprite()
        for b in self.buttons_list:
            b.display_sprite()
        clock = pygame.time.Clock()
        while self.runloop:
            clock.tick(30)
            pygame.event.pump()
            events = pygame.event.get()
            for event in events:
                if event.type is KEYDOWN and event.key is K_ESCAPE:
                    self.runloop = False
                else:
                    self.actives.update(event)
        self.dim.undim()
        self.screen.set_clip(self.scrclip)
        
                
    def set_level(self, level):
        if level < self.start_level:
            return
        for i in range(0, level):
            self.starlist[i].image = self.imgstar1
            self.starlist[i].display_sprite()
        for i in range(level, self.maxlevels):
            self.starlist[i].image = self.imgstar0
            self.starlist[i].display_sprite()



# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zytkiewicz stas.zytkiewicz@schoolsplay.org
#
#           dltr.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation. A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


####### IMPORTANT READ THIS ####################################################
# This is a special activity which provides a so called "daily training"       #
# This module looks like a regular activity but isn't.                         #
# Make sure you contact the maintainers(s) before making changes               #
################################################################################


#create logger, logger was setup in SPLogging
import logging
# In your Activity class -> 
# self.logger =  logging.getLogger("childsplay.video.Activity")
# self.logger.error("I don't understand logger")
# See SP manual for more info 

module_logger = logging.getLogger("childsplay.dltr")
import textwrap
import time
import random
from pygame.constants import *
import sqlalchemy as sqla
import utils
from SPConstants import *
import SPSpriteUtils
import SPWidgets

class ProgressBar:
    """Displays a progressbar.
    TODO: add more info"""
    def __init__(self, scr, steps, red, green):
        self.screen = scr
        self.red = red
        self.green = green
        self.start, self.end = (0, 75),(800, 75)
        self.steps = steps
        self.compleetpart = (self.end[0] - self.start[0]) / steps
        self.partsdone = 0
        
    def refresh_sprites(self):
        self._draw()

    def update(self):
        if self.partsdone == self.steps:
            return
        self.partsdone += 1
        self._draw()
    
    def _draw(self):
        done = self.partsdone * self.compleetpart
        green = self.green.subsurface(0,0,done,24)
        self.screen.blit(self.red, (self.start))
        self.screen.blit(green, (self.start))
        pygame.display.update()


class Activity:
    """  Base class mandatory for any SP activty.
    The activity is started by instancing this class by the core.
    This class must at least provide the following methods.
    start (self) called by the core before calling 'next_level'.
    post_next_level (self) called once by the core after 'next_level' *and* after the 321 count. 
    next_level (self,level) called by the core when the user changes levels.
    loop (self,events) called 40 times a second by the core and should be your 
                      main eventloop.
    get_helptitle (self) must return the title of the game can be localized.
    get_help (self) must return a list of strings describing the activty.
    get_helptip (self) must return a list of strings or an empty list
    get_name (self) must provide the activty name in english, not localized in lowercase.
    stop_timer (self) must stop any timers if any.
  """

    def __init__(self,SPGoodies):
        """SPGoodies is a class object that SP sets up and will contain references
        to objects, callback methods and observers
        TODO: add more explaination"""
        self.logger =  logging.getLogger("childsplay.dltr.Activity")
        self.logger.info("Activity started")
        self.SPG = SPGoodies
        self.scoredisplay = self.SPG.get_scoredisplay()
        self.theme = self.SPG.get_theme()
        self.screen = self.SPG.get_screen()
        self.screenclip = self.SPG.get_screenclip()
        self.blit_pos = self.screenclip.left, self.screenclip.top
        self.orgscreen = pygame.Surface(self.screenclip.size) # we use this to restore the screen
        self.orgscreen.blit(self.screen, (0, 0), self.screenclip)
        self.backgr = self.SPG.get_background()
        # The location of the activities Data dir
        self.my_datadir = os.path.join(self.SPG.get_libdir_path(),'CPData','dltrData')
        # Location of the CPData dir which holds some stuff used by multiple activities
        self.CPdatadir = os.path.join(self.SPG.get_libdir_path(),'CPData')
        # Location of the alphabet sounds dir
        self.absdir = self.SPG.get_absdir_path()# alphabet sounds dir
        self.rchash = utils.read_rcfile(os.path.join(self.my_datadir, 'dltr.rc'))
                    
        # You MUST call SPInit BEFORE using any of the SpriteUtils stuff
        # it returns a reference to the special CPGroup
        self.actives = SPSpriteUtils.SPInit(self.screen,self.backgr)
        
        self.runme = True
        self.sessionresults = utils.OrderedDict()
        self.sessionresults_raw = utils.OrderedDict()
        self.backsquare = utils.load_image(os.path.join(self.my_datadir,'background.png'))
        
        self.sessionid = time.time()

    def _set_up(self, mapper):
        """called by the core after this module is constructed."""
        self.logger.debug("Trying to fetch dbase dt data for user:%s" % mapper.currentuser)  
        
        orm = self.SPG.get_current_user_dbrow()
        self.actdatahash = self._get_data(orm.dt_target)# returns a ordereddict object
        if self.actdatahash.has_key('fortune'):
            self.fortune = True
            del self.actdatahash['fortune']
        else:
            self.fortune = False
        self.actdata = self.actdatahash.items()
        
        steps = len(self.actdata) 
        green = utils.load_image(os.path.join(self.my_datadir,'status_green_800_600.png'))
        red = utils.load_image(os.path.join(self.my_datadir,'status_red_800_600.png'))
        self.PB = ProgressBar(self.screen, steps, red, green)
        
        if self.theme == 'braintrainer' and mapper.currentuser.lower() == 'demo':
            self.logger.debug("User is demo, no prev data used.")
            self.prev_data = None
            return
        
        orm = mapper.get_orm()
        session = mapper.get_session()
        query = session.query(orm)
        results = [row.epoch for row in query.all()]
        if results:
            epoch = max(results)
        else:
            self.prev_data = None
            return
        query = query.filter(orm.user_id == mapper.user_id).filter(orm.epoch == epoch)
        rows = [row for row in query.all()]
        if not rows:
            self.prev_data = None
            return
        self.prev_data = {}
        for row in rows:
            if not row.activity or not row.done:
                self.logger.warning("No activity col or no done col found in %s" % row)
                # Fixme: in 2.2 we should remove this row from dbase (ticket 394)
                continue
            hash = {'score':row.score, 'level':row.level, 'epoch':row.epoch}
            self.prev_data[row.activity] = hash
    
    def _get_data(self, target):
        self.logger.debug("Getting data for target %s" % target)
        acthash = utils.OrderedDict()
        orm, session = self.SPG.get_orm('dt_sequence', 'user')
        rows = session.query(orm).filter(orm.target == target).order_by(orm.order).all()
        for row in rows:
            hash = {}
            hash['fortune'] = row.fortune
            hash['name'] = row.act_name
            hash['group'] = row.group
            hash['level'] = row.level
            hash['cycles'] = row.cycles
            hash['target'] = row.target
            hash['order'] = row.order
            acthash[row.act_name] = hash
        self.logger.debug("Found data: %s" % acthash)
        return acthash

    def clear_screen(self):
        self.screen.blit(self.orgscreen,self.blit_pos)
        self.backgr.blit(self.orgscreen,self.blit_pos)
        pygame.display.update()

    def get_moviepath(self):
        movie = os.path.join(self.my_datadir,'help.avi')
        return movie

    def refresh_sprites(self):
        """Mandatory method, called by the core when the screen is used for blitting
        and the possibility exists that your sprites are affected by it."""
        self.actives.refresh()
        self.PB.refresh_sprites()

    def get_helptitle(self):
        """Mandatory method"""
        return "dltr"
    
    def get_name(self):
        """Mandatory method, returnt string must be in lowercase."""
        return "dltr"
    
    def get_help(self):
        """Mandatory methods"""
        text = [_("The aim of this activity:"),
        _("This activity is known as the daily training which is a collection of activities."), 
        " "]
        return text 
    
    def get_helptip(self):
        """Mandatory method, when no tips available returns an empty list"""
        return []
        
    def get_helptype(self):
        """Mandatory method, you must set an type"""
        # Possible types are: Memory, Math, Puzzle, Keyboardtraining, Mousetraining
        #                     Language, Alphabet, Fun, Miscellaneous
        return _("Miscellaneous")
        
    
    def get_helplevels(self):
        """Mandatory method, must return a string with the number of levels
        in the follwing format:
        _("This level has %s levels") % number-of-levels"""
        #return _("This activity has %s levels") % 1
        #DAILY TRAINING HAS NO LEVELS
        return " " 
    
    def _cbf(self, *args):
        return args
    
    def pre_level(self,level):
        """Mandatory method.
        Return True to call the eventloop after this method is called."""
        self.logger.debug("pre_level called with: %s" % level)
        if not self.runme:
            return
        self.SPG.tellcore_disable_level_indicator()
        self.SPG.tellcore_disable_score_button()
        self.screen.blit(self.backsquare, (50, 110))
        txt = [_("If you ready to start the next activity, hit the 'start' button.")]
        txt = utils.txtfmt(txt, 40)
        y = 200
        for t in txt:
            surf = utils.char2surf(t,28,WHITE)
            r = self.screen.blit(surf, (80, y))
            pygame.display.update(r)
            y += 50
        startbutpath = utils.load_image(os.path.join(self.my_datadir, 'start.png'))
        startbutpath_ro = utils.load_image(os.path.join(self.my_datadir, 'start_ro.png'))
        self.startbutton = SPWidgets.TransImgButton(startbutpath, startbutpath_ro, \
                                    (300, 350), fsize=32, text= _("Start"), fcol=WHITE)
        self.startbutton.connect_callback(self._cbf, MOUSEBUTTONDOWN, 'start')
        self.startbutton.set_use_current_background(True)
        self.startbutton.display_sprite()
        self.actives.add(self.startbutton)
        self.start_loop_flag = True
        self.PB.update()
        return True 
        
    def start(self):
        """Mandatory method."""
        try:
            self.currentactname, self.currentactdata = self.actdata.pop(0)
        except IndexError:
            self.logger.debug("No more activities to run")
            self.runme = False
            self.clear_screen()
        else:
            self.logger.debug("activity %s is next." % self.currentactname)
        self.act_score = 0
    
    def act_stopped(self, activity):
        """called by the core when an act has finished the levels."""
        self.logger.debug("act_stopped called")
        self.act_score = self.act_score / int(self.currentactdata['cycles'])
        stime = utils.calculate_time(self.act_start_time, \
                utils.current_time())
        self.dbmapper.insert('start_time', self.act_start_time)
        self.dbmapper.insert('end_time', utils.current_time())
        self.dbmapper.insert('timespend', stime)
        self.dbmapper.insert('score', self.act_score)
        self.dbmapper.insert('done', 1)
        self.dbmapper.commit()
        # TODO: when we have a proper dbase setup we can get the results from the dbase
        # but for now we keep it local for use in the end display.
        self.sessionresults[activity.get_helptitle()] = '%4.2f' % self.act_score
        self.sessionresults_raw[activity.get_name()] = self.act_score

    def act_next_level_stopped(self, result):
        """called by the core when the act finished a level.
        This is also called after the last level of act, just before the core calls
        act_stopped."""
        self.logger.debug("act_next_level_stopped called with: %s" % result)
        self.act_score = self.act_score + result
        self.logger.debug("activity score now %s" % self.act_score)
        
    
    def act_hit_levelindicator(self, *args):
        """NOT USED YET
        Called by the core when the levelindicator is hit by the user to change
        the level. (If we enabled the levelindicator of course)
        We use this to determine if we switch acts when act_stopped is called or that
        we must restart with all acts from the new level."""
        pass
    def dt_restarting(self):
        """Called just before the core restarts the DT"""
        pass

    def get_actdata(self):
        """Called by the core"""
        return self.currentactdata
    
    def next_activity(self):
        self.logger.debug("next_activity called")
        if self.prev_data and self.prev_data.has_key(self.currentactname):    
            self.logger.debug("previous act data for %s: %s" % \
                          (self.currentactname, self.prev_data[self.currentactname]))
            try:
                if self.prev_data[self.currentactname]['score'] < 6.5 and\
                    self.prev_data[self.currentactname]['level'] > 1:
                    level = self.prev_data[self.currentactname]['level'] - 1
                elif self.prev_data[self.currentactname]['score'] > 8.0 and\
                    self.prev_data[self.currentactname]['level'] < 6:
                    level = self.prev_data[self.currentactname]['level'] + 1
                else:
                    level = self.prev_data[self.currentactname]['level']
            except Exception, msg:
                self.logger.warning("Failed to query previous dt data: %s" % msg)
                self.logger.warning("Possible dbase corruption, prevdata was: %s" % self.prev_data)
                self.logger.warning("setting level to 2 and continue")
                level = 2
            self.logger.debug("old level: %s new level: %s" % \
                            (self.prev_data[self.currentactname]['level'], level))
        else:
            self.logger.warning("No prevdata found for %s, getting level from current data" % self.currentactname)
            level = self.actdatahash[self.currentactname]['level']
        cycles = self.currentactdata['cycles']
        self.dbmapper.insert('cycles', cycles)
        self.dbmapper.insert('level', level)
        self.act_start_time = utils.current_time()
        self.dbmapper.insert('activity', self.currentactname)
        self.SPG.tellcore_show_level_indicator()
        self.SPG.tellcore_enable_score_button()
        self.SPG._menu_activity_userchoice(self.currentactname, level, cycles)

    def next_level(self,level,dbmapper):
        """Mandatory method.
        Return True if there levels left.
        False when no more levels left."""
        if not self.runme:# in case of an error or when there are no act left this flag is set
            self.logger.debug("DT ends, no more acts")
            self.display_results()            
            return False
        self.logger.debug("nextlevel called with: %s" % level)
        self.SPG.tellcore_disable_level_indicator()
        #self.SPG.tellcore_hide_level_indicator()
        
        self.dbmapper = dbmapper
        self.dbmapper.insert('epoch', self.sessionid)
        self.next_activity()
        return True

    def display_results(self):
        txt0 = _("You have finished the daily training module.")
        txt1 = _("Your results are:")
        txt1a = _("Your average results per group are:")
        txt2 = _("Activity:")
        txt3 = _("Score:")
        # positions
        txt0_pos = (10, 4)
        txt1_pos = (10, 30)
        txt2_pos = (10, 66)
        txt3_pos = (300, txt2_pos[1])
        
        # user screen
        groups = {}
        for name, data in self.actdatahash.items():
            group = data['group']
            score = self.sessionresults_raw[name]
            if not groups.has_key(group):
                groups[group] = (0, 0)
            groups[group] = (groups[group][0] + score, groups[group][1] + 1)  
        
        surf = self.backsquare.convert_alpha()
        s = utils.char2surf(txt0, 20, WHITE)
        surf.blit(s, txt0_pos)
        s = utils.char2surf(txt1a, 20, WHITE)
        surf.blit(s, txt1_pos)
        s = utils.char2surf(txt2, 20, WHITE)
        surf.blit(s, txt2_pos)
        s = utils.char2surf(txt3, 20, WHITE)
        surf.blit(s, txt3_pos)
        y = txt2_pos[1] + 46
        
        for name, data in groups.items():
            score = data[0] / data[1]
            sk = utils.char2surf(name.capitalize(), 20, WHITE) 
            sc = '%4.2f' % score
            if score < 6:
                # always just above 6 :-)
                sc = '%4.2f' % (6 + (random.random()/3))
            sv = utils.char2surf(sc, 20, WHITE)
            surf.blit(sk, (txt2_pos[0], y))
            surf.blit(sv, (txt3_pos[0], y))
            y += 26
        
        if self.fortune:
            lang = utils.get_locale_local()[0].split('_')[0]
            orm, session = self.SPG.get_orm('game_languages', 'content')
            query = session.query(orm).filter_by(lang = lang)
            lang = [l.ID for l in query.all()]
            session.flush()
            session.close()
            # Disabled the fortune display until we decided what to do with it.
#            orm, session = self.SPG.get_orm('game_quotes', 'content')
#            query = session.query(orm).filter(orm.language.in_(lang))
#            fortune = random.choice([result.quote for result in query.all()])
#            y = 300
#            for line in textwrap.wrap(fortune, 40):
#                s = utils.char2surf(line, 20, WHITE)
#                surf.blit(s, (40, y))
#                y += s.get_rect().h
        
        dlg = SPWidgets.Dialog(SPWidgets.Widget(surf),buttons=[_("Details"), _("OK")], title=_('Results'))
        dlg.run() # this blocks any other events loops
        answer = dlg.get_result()
        dlg.erase_sprite()
        if answer[0] == _("OK"):
            return
            
        # details screen
               
        surf = self.backsquare.convert_alpha()
        s = utils.char2surf(txt0, 20, WHITE)
        surf.blit(s, txt0_pos)
        s = utils.char2surf(txt1, 20, WHITE)
        surf.blit(s, txt1_pos)
        s = utils.char2surf(txt2, 20, WHITE)
        surf.blit(s, txt2_pos)
        s = utils.char2surf(txt3, 20, WHITE)
        surf.blit(s, txt3_pos)
        
        y = txt2_pos[1] + 26
        for k, v in self.sessionresults.items():
            sk = utils.char2surf(k, 20, WHITE)
            sv = utils.char2surf(v, 20, WHITE)
            surf.blit(sk, (txt2_pos[0], y))
            surf.blit(sv, (txt3_pos[0], y))
            y += 46
        
        self.SPG.tellcore_info_dialog(SPWidgets.Widget(surf))

    def post_next_level(self):
        """Mandatory method.
        This is called once by the core after 'next_level' *and* after the 321 count.
        You should place stuff in here that run in a seperate thread like sound play."""
        pass 
        
    def get_score(self,timespend):
        """Mandatory method.
        @timespend is the time spend inside the level as it's calculated
        by the core.
        returns the score value for the past level.
        return None if no score value is used"""
        pass
    
    def stop_timer(self):
        """You *must* provide a method to stop timers if you use them.
        The SPC will call this just in case and catch the exception in case 
        there's no 'stop_timer' method""" 
        try:
            self.timer.stop()
        except:
            pass
        
    def loop(self,events):
        """Mandatory method.
        This is the main eventloop called by the core 30 times a minute."""
        for event in events:
            if event.type in (MOUSEBUTTONDOWN, MOUSEMOTION):
                if self.actives.update(event):
                    self.logger.debug("Starting %s" % self.currentactname)
                    self.startbutton.erase_sprite()
                    self.actives.empty()
                    self.SPG.tellcore_pre_level_end()
                    self.clear_screen()
        return 
        



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2010-2011 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           text.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from base import Widget
from SPConstants import *
from SPSpriteUtils import SPSprite
import utils
import pygame
from pygame.constants import *
import types
from funcs import render_textrect
if not NoGtk:
    import pangofont
    
class Label(Widget):
    def __init__(self, txt, pos, fsize=18, padding=4, border=None, maxlen=0, name='', bold=False, transparent=False, ** kwargs):
        """Label which displays a line of text
        txt - string to display
        pos - position to display the box
        rect - Rect indicating the size of the box
        fsize - Font size
        padding - space in pixels around the text
        border - Integer. Draw a border around the label.
        name - string to indicate this object
        bold - set font to bold.
        
        Theres also a number of keyword arguments understood by this widget;
        fgcol - foreground color (other than the theme color)
        bgcol - Background color.
        minh - Minimal height of the widget.
        transparent - No background.
        """ 
        Widget.__init__(self)
        self.fsize = fsize
        self.bold=bold
        self.transparent = transparent
        fgcol = self.THEME['label_fg_color']
        bgcol = self.THEME['label_bg_color']
        if kwargs.has_key('fgcol'):
            fgcol = kwargs['fgcol']
        if kwargs.has_key('bgcol'):
            bgcol = kwargs['bgcol']
        if kwargs.has_key('minh'):
            self.minh = kwargs['minh']
        else:
            self.minh = None
        if kwargs.has_key('ttf'):
            self.ttf = kwargs['ttf']
        else:
            self.ttf = TTF
        if padding < 2:
            padding = 2
        self.padding = padding
        self.fgcol = fgcol
        self.bgcol = bgcol
        self.border = border
        self.settext(txt)
        self.rect  = self.image.get_rect()
        SPSprite.__init__(self, self.image)
        self.rect.move_ip(pos)
        
    def settext(self, txt):
        self._txt = txt
        s = utils.char2surf(txt, fsize=self.fsize,ttf=self.ttf, fcol=self.fgcol, bold=self.bold)
        r = s.get_rect()
        if self.minh:
            y = self.minh
            x = r.w
            x += self.padding*2
            surf = pygame.Surface((x, y))
        else:
            x = r.w
            x += self.padding*2
            y = r.h
            y += self.padding*2
            surf = pygame.Surface((x, y))
        x = (surf.get_rect().w - r.w) / 2
        y = (surf.get_rect().h - r.h) / 2
        surf.fill(self.bgcol)
        surf.blit(s, (x, y))
        if self.transparent:
            self.image = s
        else:
            self.image = surf
        if self.border:
            pygame.draw.rect(self.image, BLACK, self.image.get_rect(), self.border)
    def get_text(self):
        return self._txt


class TextView(Widget):
    def __init__(self, txt, pos, rect=None, fsize=12, padding=4, \
                 autofit=False, border=False, name='', \
                 bgcol=None, fgcol=None, shade=0, bold=False, **kwargs):
        """Box which displays a text, wrapped if needed. 
        txt - string to display. When txt is a list of strings the lines are
            blitted on a surface big enough to hold the longest line.
            rect is not used in this case.
        pos - position to display the box
        rect - Rect indicating the size of the box. 
                Only used in conjunction with txt if it's a string.
        fsize - Font size
        padding - space in pixels around the text
        fgcol - text color (R,G,B), if None the theme color will be used.
        bgcol - background color (R,G,B), if None the theme color will be used.
                If bgcol is set to 'trans' the background will be transparent.
        autofit - Discard the empty space around the text.
        name - string to indicate this object
        shade - The amount of "3D shade" the text should have, 0 means no shade.
                Beware that shading takes time.
        """
        if not fgcol: 
            fgcol = self.self.THEME['textview_fg_color']
        if not bgcol:
            bgcol = self.self.THEME['textview_bg_color']
        if type(txt) in types.StringTypes and rect != None:
            self.image = render_textrect(txt, fsize, TTF, rect, fgcol, \
                              bgcol, justification=0, bold=bold, \
                              autofit=autofit, border=border)
        elif type(txt) is types.ListType:
            ll = []
            w = 0
            for line in txt:
                if not line.strip('\n'):
                    line = ' '
                if shade:
                    s = utils.shadefade(line, fsize, amount=shade, bold=bold, fcol=fgcol, shadecol=DARKGREY)
                    ll.append(s)
                else:
                    s = utils.char2surf(line, fsize, fgcol, bold=bold)
                    ll.append(s)
                if s.get_rect().w > w:
                    w = s.get_rect().w
                
            h = s.get_rect().h   
            if bgcol == 'trans':
                self.image = pygame.Surface((w, h * len(ll)), SRCALPHA)
                self.image.fill((0, 0, 0, 0))
            else:
                self.image = pygame.Surface((w, h * len(ll)))
                self.image.fill(bgcol)
            x, y = 0, 0
            for s in ll:
                self.image.blit(s, (x, y))
                y += h
            
        self.rect  = self.image.get_rect()
        Widget.__init__(self, self.image)
        self.rect.move_ip(pos)       

class SimpleView(Widget):
    """Simple object that displays a numbers of strings.
        Any strings that are longer than the widgets rect are disgarded.
        Use this as a view for command that produces text output repeatedly
        like a shell command.
        Use the TextView for proper text display."""
    def __init__(self, rect, fsize, bgcol=None, fgcol=None, padding=8, border=2, lines=1):
        """Starts empty, use set_text to fill it.
        rect - Rect indicating the size of the box and position. 
        fsize - Font size
        fgcol - text color (R,G,B), if None the theme color will be used.
        bgcol - background color (R,G,B), if None the theme color will be used.
                If bgcol is set to 'trans' the background will be transparent.
        padding - space in pixels around the text
        border - border in pixels, None means no border.
        lines - how many previous lines of text should be visible (cheap scroll effect)
                """
        if not fgcol: 
            fgcol = self.THEME['textview_fg_color']
        if not bgcol:
            bgcol = self.THEME['textview_bg_color']
        self.fgcol = fgcol
        self.bgcol = bgcol
        self.padding = padding
        self.fsize = fsize
        self.image = pygame.Surface(rect.size)
        self.image.fill(self.bgcol)
        self.rect = self.image.get_rect()
        if border:
            pygame.draw.rect(self.image, BLACK, self.rect, border)
        self.org_image = self.image.copy()
        Widget.__init__(self, self.image)
        self.rect.move_ip(rect.topleft)
        self.prevlines = []
        self.lines = lines
    
    def set_text(self, text):
        """Add one line of text."""
        self.prevlines.append(text)
        if len(self.prevlines) > self.lines:
            self.prevlines = self.prevlines[-self.lines:]
        self.image.blit(self.org_image, (0, 0))
        y = 0
        for line in self.prevlines:
            surf = utils.char2surf(line, self.fsize, self.fgcol)
            self.image.blit(surf, (self.padding, self.padding + y))
            y += surf.get_height()

class TextEntry(Widget):
    def __init__(self, pos, length=200, maxlen=0, message='', fsize = 28, border=None, \
                 bold=None, ttf='arial', fgcol=None, bgcol=None, password_mode=False, \
                 validationlist=[], validation_values_list=[]):
        """Box which displays a text input. The height of the box is determined
        by the fontsize used.
        message - input prompt
        pos - position to display the box
        maxlen - is the maximum number of input characters.
        length - length of the display in pixels.
        fsize - font size
        border - One pixel border around the box
        ttf - family of fonts
        fgcol - foreground color (defaults to theme)
        bgcol - background color (defaults to theme)
        password_mode - displays input in stars
        validitionlist - A list with characters that are allowed
        validation_values_list - A list with values that are allowed
        """
        Widget.__init__(self)
        self.logger = logging.getLogger("childsplay.SPWidgets_lgpl.TextEntry")
        self.length = length
        self.maxlen = maxlen
        self.fsize = fsize
        self.message = message
        self.border = border
        self.ttf = ttf
        self.fgcol = fgcol
        if not self.fgcol:
            self.fgcol = self.THEME['textentry_fg_color']
        self.bgcol = bgcol
        if not self.bgcol:
            self.bgcol = self.THEME['textentry_bg_color']
        self.bold = bold
        self.password_mode = password_mode
        self.validationlist = validationlist
        self.validation_values_list = validation_values_list
        if NoGtk:
            font = pygame.font.Font(filename=None, size=fsize)
        else:
            font = pangofont.PangoFont(family=ttf, size=fsize, bold=bold)
        w, h = font.size(message)
        self.image = pygame.Surface((self.length, h))
        self.rect = self.image.get_rect()
        self.image.fill(self.bgcol)
        if self.border:
            pygame.draw.rect(self.image, BLACK, self.image.get_rect(), self.border)
        self.orgimage = self.image.convert()
        self._CanHaveInputFocus = True
        self.connect_callback(self._cbf, [KEYDOWN, MOUSEBUTTONDOWN])
        self.moveto(pos)
        self.xpadding = 6
        self.ypadding = 0
        if self.message:
            self._remove_prompt()
        
    def _cbf(self, widget, event, data):
        if event.type == MOUSEBUTTONDOWN:
            self._add_prompt()
        elif event.type == KEYDOWN:
            if event.key == K_BACKSPACE:
                self.backspace()
            elif event.key == K_RETURN:
                return self.get_text()
            elif event.unicode and ord(event.unicode) > 31:
                if self.validationlist:
                    if event.unicode in self.validationlist:
                        self.add(event.unicode)
                else:
                    self.add(event.unicode)
    
    def _add_prompt(self):
        self.image.blit(self.orgimage, (0, 0))
        self.draw()
    
    def _remove_prompt(self):
        self.image.blit(self.orgimage, (0, 0))
        self.messagesurf = utils.char2surf(self.message, self.fsize, self.fgcol, bold=self.bold)
        self.image.blit(self.messagesurf, (self.xpadding, self.ypadding))
        self.display_sprite()

    def add(self, c):
        if self.maxlen and len(self.message) > self.maxlen-1:
            return
        self.message += c
        self.draw()
        
    def backspace(self):
        self.message = self.message[:-1]
        self.draw()
    
    def draw(self):
        if self.password_mode:
            self.messagesurf = utils.char2surf('*' * len(self.message) +'_', self.fsize, self.fgcol, bold=self.bold)
        else:
            self.messagesurf = utils.char2surf(self.message+'_', self.fsize, self.fgcol, bold=self.bold)
        if self.messagesurf.get_size()[0] <= self.length:
            self.image.blit(self.orgimage, (0, 0))
            self.image.blit(self.messagesurf, (self.xpadding, self.ypadding))
        else:
            self.message = self.message[:-1]
        self.display_sprite()
     
    def clear(self):
        self.image.blit(self.orgimage, (0, 0))
        self.message = ""
        self.display_sprite()
        
    def get_text(self):
        txt = self.message.strip()
        try:
            i = int(txt)
            txt = str(i)
        except ValueError:
            pass
        if self.validation_values_list:
            if txt in self.validation_values_list:
                return txt
            else:
                return ''
        return txt
    
    def set_text(self,txt):
        self.message = txt
        self._remove_prompt() 
        
    def get_surface(self):
        return self.image

class TEB_TextEntry(TextEntry):
    """This eidget is used internally by the TextEntryBox. 
    Don't use this in your code, use the TextEntry
    """
    __current = None
    def __init__(self, pos, length=200, maxlen=0, message='', fsize = 28, border=None, \
                 bold=None, ttf='arial', fgcol=None, bgcol=None, password_mode=False):
        TextEntry.__init__(self, pos, length=length, maxlen=maxlen, \
                           message=message, fsize=fsize, border=border, \
                        bold=bold, ttf=ttf, fgcol=fgcol, bgcol=bgcol,\
                        password_mode=password_mode)
        self.logger = logging.getLogger("childsplay.SPWidgets_lgpl.TEB_TextEntry")
        self._TEfocus = False
    
    def _cbf(self, widget, event, data):
        pass
   
        
class TextEntryBox(Widget):
    def __init__(self, pos, maxlen=0, length=300, height=2, message='', fsize = 14, border=True, \
                 ttf='arial', fgcol=None, bgcol=None, bold=False, password_mode=False):
        """Text input box with multiple lines. The height of the box is determined
        by the fontsize used times the height. You can get the actual height in pixels by calling
        the "get_height" method.
        message - input prompt
        pos - position to display the box
        maxlen - is the maximum number of input characters.
        length - length of the display in pixels.
        height - height in characters lines, 2 means two lines of text.
        fsize - font size
        border - One pixel border around the box
        ttf - family of fonts
        """
        self.height = height
        self.TEs = utils.OrderedDict()
        self.currentline = 0
        self.maxline = height - 1
        sizey = 0
        tepos = (pos[0]+1, pos[1]+1)
        for line in range(height):
            te = TEB_TextEntry(pos=tepos, maxlen=maxlen, \
                            length=length, message=message,\
                            fsize = fsize, border=False, ttf='arial',\
                            fgcol=fgcol, bgcol=fgcol, bold=bold, \
                            password_mode=False)
            te.disconnect_callback()
            te.connect_callback(self._cbf, [KEYDOWN, MOUSEBUTTONDOWN])
            sizey += te.get_sprite_height()
            tepos = (tepos[0], tepos[1] + te.get_sprite_height())
            te.line = line
            self.TEs[line] = te
        sizex = te.get_sprite_width()# length stays the same for all lines
        self.image = pygame.Surface((sizex + 2, sizey + 2))
        Widget.__init__(self, self.image)
        self.image.fill(self.THEME['textentry_bg_color'])
        
        y = 0
        for te in self.TEs.values():
            self.image.blit(te.get_surface(), (0, y))
            y += te.get_sprite_height()
        if border:
            pygame.draw.rect(self.image, BLACK, self.rect, 1)
        self.moveto(pos)
        
    def _cbf(self, widget, event, data):
        if event.type == MOUSEBUTTONDOWN:
            if TEB_TextEntry._TEB_TextEntry__current:
                TEB_TextEntry._TEB_TextEntry__current._remove_prompt()
            TEB_TextEntry._TEB_TextEntry__current = widget
            self.currentline = widget.line
            widget._add_prompt()
        elif event.type == KEYDOWN:
            if TEB_TextEntry._TEB_TextEntry__current is not widget:
                return
            if event.key == K_BACKSPACE:
                widget.backspace()
            elif event.key == K_RETURN:
                if self.currentline < self.maxline:
                    widget._remove_prompt()
                    self.currentline += 1
                    newwidget = self.TEs[self.currentline]
                    newwidget._add_prompt()
                    TEB_TextEntry._TEB_TextEntry__current = newwidget
            elif event.unicode and ord(event.unicode) > 31:
                widget.add(event.unicode)
    
    def get_text(self):
        return [te.get_text() for te in self.TEs.values()]
        
    def get_actives(self):
        return self.TEs.values()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2010-2011 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           base.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from SPSpriteUtils import SPSprite
import os
import types
import subprocess
import glob
import ConfigParser
from SPConstants import *
import logging 
module_logger = logging.getLogger("childsplay.SPWidgets")

WEHAVEAUMIX = False
THEME = None

def Init(theme):
    """Must be called before using anything in this class"""
    global THEME,  WEHAVEAUMIX
    module_logger.debug('Init called with:%s' % theme)
    # theme is used by the widgets
    config = ConfigParser.RawConfigParser()
    rc = os.path.join( GUITHEMESPATH, theme, 'SPWidgets.rc')
    if not os.path.exists(rc):
        module_logger.info('theme file %s does not exists' % rc)
        rc = os.path.join('..', GUITHEMESPATH, theme, 'SPWidgets.rc')
        module_logger.info('trying one step higher, %s' % rc)
        if not os.path.exists(rc):
            module_logger.info('theme file %s does not exists, using default file' % rc)
            rc = os.path.join(DEFAULTGUITHEMESPATH,'SPWidgets.rc')
    module_logger.debug("Using rc file %s" % rc)
    d = {}
    config.read(rc)
    for k, v in dict(config.items('default')).items():
        try:
            d[k] = eval(v, {'__builtins__': None}, {})
        except NameError:
            # v is a string
            d[k] = v
    d['theme'] = theme
    d['themepath'] = os.path.dirname(rc)
    d['defaultpath'] = os.path.join( GUITHEMESPATH,'default')
    module_logger.debug("rc file contents: %s" % [d])     
    THEME = d
    volume_level = 50
    try:
        cmd=subprocess.Popen("amixer get Master",shell=True,\
                              stdout=subprocess.PIPE,\
                              stderr=subprocess.PIPE)
        output = cmd.communicate()[0]
    except Exception, info:
        module_logger.warning("program 'amixer' not found, unable to set volume levels: %s" % info)
        WEHAVEAUMIX = False
    else:
        for line in output.split('\n'):
            if "%]" in line:
                volume_level = line.split("%]")[0].split("[")[1]
        WEHAVEAUMIX = volume_level
    return THEME

class Widget(SPSprite):
    def __init__(self, image=None, name=None):
        global THEME, WEHAVEAUMIX
        if image:
            SPSprite.__init__(self, image, name=name)
        else:
            SPSprite.__init__(self, name=name)
        self._mygroups = None
        self._amienabled = True
        self.trans = False
        self._oldimage = None
        self._IamWidget = True #special property, used in the SPSprite class update method
        self._CanHaveInputFocus = False
        self.MinimalDragTreshold = 20
        self._DragEnabled = False
        self._DropEnabled = False
        self.mouse_hover_leave_action = False
        # related to drag and drop
        self._prev_mouse_pos = None
        self.THEME = THEME
        self.WEHAVEAUMIX = WEHAVEAUMIX
        
    def enable(self, enable):
        """Set wetter the sprite can react to events. 
        enable - True or False 
        When enable is set to False the sprite is disconnected from the callback
        function as well as 'greyed-out'.
        When enable is True the sprite is connected again and redisplayed.
        """
        if not enable and self._amienabled:
            if hasattr(self, 'mouse_hover_leave'):
                self.mouse_hover_leave()
            # TODO: find a good solution for alpha trans for trans buttons.
            # problem lies in the per pixel alpha the buttons are using.
#            if self.trans:
#                print "im a trans widget", self.name
#                self._oldimage = self.image.convert_alpha()
#                self.image.blit(self.trans, (0, 0))
#            else:
#                self.image.set_alpha(80)
            self.image.set_alpha(80)
            self._mygroups = self.groups()
            self.remove(self._mygroups)
            self.erase_sprite()
            self.display_sprite()
            self._amienabled = False
        elif enable and not self._amienabled:
            if hasattr(self, 'mouse_hover_leave'):
                self.mouse_hover_leave()
#            if self.trans and self._oldimage:
#                self.image = self._oldimage.convert_alpha()
#            else:
#                self.image.set_alpha(255)
            self.image.set_alpha(255)
            if self._mygroups:
                self.add(self._mygroups)
            self.erase_sprite()
            self.display_sprite()
            self._amienabled = True
            
    def set_drag_enabled(self, enabled):
        self._DragEnabled = enabled
        self._org_center = self.rect.center
    def set_drop_enabled(self, enabled):
        self._DropEnabled = enabled
    def get_actives(self):
        return self
    
    def _DnDreset(self):
        self.moveto(self._org_pos,False)
        
    # These methods should be overridden in your derived class
    def start_drag_event(self, widget, mouseposition):
        self.logger.debug("start_drag_event: widget %s, position %s" % (widget,mouseposition))
    def end_drag_event(self, widget, mouseposition):    
        self.logger.debug("end_drag_event: widget %s, position %s" % (widget,mouseposition))
    def drop_event(self, widget, mouseposition):
        self.logger.debug("end_drag_event: widget %s" % widget,mouseposition)
    


# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           SPlogCheck.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os, re, glob
from SPConstants import SPLC_TIMESTAMP, HOMEDIR
FILEPATH = os.path.join(HOMEDIR, "SPErrorLog_" + SPLC_TIMESTAMP +".log")
from SPLogging import LOGPATH
from utils import SPError, MyError

import logging
SPLCmodule_logger = logging.getLogger("childsplay.SPlogCheck")
SPLCmodule_logger.debug("Starting to parse SP log %s for warning messages and higher" % LOGPATH)

oldfiles = glob.glob(os.path.join(HOMEDIR, "SPErrorLog_*"))
oldfiles.sort()
if len(oldfiles) > 20:
    SPLCmodule_logger.warning("Removing older error files")
    for i in range(20, len(oldfiles)):
        try:
            f = oldfiles.pop(0)
            os.remove(f)
        except (IOError, IndexError):
            SPLCmodule_logger.warning("Can't remove file %s" % f)
        else:
            SPLCmodule_logger.warning("removed %s" % f)
            
# Which messages to look for
pattern = re.compile("ERROR|CRITICAL")
lines = []
try:
    f = open(LOGPATH, 'r')
    #f = open('/home/stas/.schoolsplay.rc/test.log', 'r')
    lines = f.readlines()
except IOError:
    SPLCmodule_logger.warning("Can't find the SP logfile ?")
    SPLCmodule_logger.warning("Start schoolsplay with minimal loglevel info to get a logfile.")
    SPLCmodule_logger.warning("Stopping the log parsing.")

if not lines:
    SPLCmodule_logger.warning("The logfile %s hasn't any content ?" % LOGPATH)
    SPLCmodule_logger.warning("Stopping the log parsing.")
    lines =[]
    
notifycaller = False
for line in lines:
    if re.search(pattern, line):
        SPLCmodule_logger.warning("Found error messages in the log file, please contact the developers")
        SPLCmodule_logger.warning("Saving the SP log to %s" % FILEPATH)
        f = open(FILEPATH, 'w')
        f.writelines(lines)
        f.close()
        notifycaller = True
        break
        
if notifycaller:        
    raise MyError, _("Found error messages in the log file, please contact the developers.")






# -*- coding: utf-8 -*-

# Copyright (c) 2006 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           utils.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import __builtin__
import imp
import sys
from SPConstants import *
from NormalDistTable import normal_distribution_table
import gettext
import glob
import fnmatch
import locale
import logging
import os
if not NoGtk:
    import pangofont
import pygame
import shutil
import time
import datetime
import types
import ConfigParser
import math
from collections import MutableMapping
from pygame.constants import *
from pygame import surfarray
import textwrap
#from facedetect.facedetect import CAMFOUND
CAMFOUND = None
if CAMFOUND:
    import opencv.adaptors as adaptors
module_logger = logging.getLogger("childsplay.utils")

if NoGtk:
    module_logger.warning("NoGtk is set, not using GTK pango font rendering some non-latin characters might be missing.")


UT_DEBUG = 0

class StopGameException(Exception):
    # Raise this to quit outside the eventloop
    pass

class MyError(Exception):
    def __init__(self, str='', line=''):
        self.line = line
        self.str = str
    def __str__(self): return "%s" % self.str

class SPError(Exception):
    pass
class NoSuchTable(SPError):
    pass
class ReporterError(SPError):
    pass
class XMLParseError(SPError):
    pass
class StopmeException(Exception):
    pass
class RestartMeException(Exception):
    pass
class TextRectException(Exception):
    def __init__(self, message = None):
        self.message = message
    def __str__(self):
        return self.message

###### For debugging memoryleaks #####################
#def memory_profile(h):
#    f = open('heap_profile.txt', 'a')
#    f.write('heap: \n%s' % h.heap())
#    f.write('\n\n')
#    hh = h.heap()
#    #f.write('heap byvia: \n%s' % hh.byvia)
#    f.write('Traversing Heap: \n%s' % hh.get_rp())
#    f.write('\n\n')
#    f.close()
#####################################################

def calcdist(v1, v2):
    """Calculate the distance between two vectors where a vector is a tuple (x,y)"""
    dx = v2[0] - v1[0]
    dy = v2[1] - v1[1]
    return math.sqrt(dx*dx + dy*dy)

def set_locale(lang=None):
    """Used by the core to set the locale.
    """
    module_logger.debug("set_locale called with %s" % lang)
    global LOCALE_RTL, LANG # These come from SPConstants
    txt = ""
    try:
        if not lang or lang == 'system':
            try:
                # FIX locale.py LANGUAGE parsing bug, the fix was added on the
                # upstream CVS on the 1.28.4.2 revision of 'locale.py', It
                # should be included on Python 2.4.2.
                if os.environ.has_key('LANGUAGE'):
                    lang = os.environ['LANGUAGE'].split(':')[0]
                else:
                    lang, enc = locale.getdefaultlocale()
                    lang = "%s.%s" % (lang, enc.lower())
            except ValueError, info:
                module_logger.error(info)
                lang = 'en'
        languages = [lang]
        if os.environ.has_key('LANGUAGE'):
            languages += os.environ['LANGUAGE'].split(':')
        module_logger.info("Setting seniorplay locale to '%s' modir: %s" % (lang, LOCALEDIR))
        lang_trans = gettext.translation('childsplay', \
            localedir=LOCALEDIR, \
            languages=languages)
        __builtin__.__dict__['_'] = lang_trans.ugettext
    except Exception, info:
        txt = ""
        if lang and lang.split('@')[0].split('.')[0].split('_')[0] != 'en':
            txt = "Cannot set language to '%s' \n switching to English" % lang
            module_logger.info("%s %s" % (info, txt))
        __builtin__.__dict__['_'] = lambda x:x
        lang = 'en_US.utf8'
    else:
        lang = lang.split('@')[0]#.split('.')[0].split('_')[0]
    # This is to signal that we running under a RTL locale like Hebrew or Arabic
    # Only Hebrew and Arabic is supported until now
    if lang[:2] in ['he', 'ar']:
        LOCALE_RTL = True
    LANG = lang
    module_logger.debug("Locale set to %s, RTL set to %s" % (LANG, LOCALE_RTL))
    return (lang, LOCALE_RTL)

def get_locale_local():
    """Returns the locale set by set_locale.
    This can differ from systems locale."""
    return (LANG, LOCALE_RTL)

def get_locale():
    """Get the systems locale.
    This can be different from the locale used by childsplay.(see set_locale())"""
    global LOCALE_RTL
    try:
        lang = ''
        # FIX locale.py LANGUAGE parsing bug, the fix was added on the
        # upstream CVS on the 1.28.4.2 revision of 'locale.py', It
        # should be included on Python 2.4.2.
        if os.environ.has_key('LANGUAGE'):
            lang = os.environ['LANGUAGE'].split(':')[0].split('_')[0]
        # This makes sure that we never return a value of None.
        # This is a fix for systems that set LANGUAGE to ''.
        if lang == '':
            lang = locale.getdefaultlocale()[0].split('_')[0]
    except Exception, info:
        module_logger.error("%s, %s" % (info, "Switching to English"))
        lang = 'en'
    if lang == 'C' or lang.lower() == 'posix':
        # Default to English
        lang = 'en'
    # This is to signal that we running under a RTL locale like Hebrew or Arabic
    # Only Hebrew and Arabic is supported until now
    elif lang in ['he', 'ar']:
        LOCALE_RTL = True
    return (lang, LOCALE_RTL)

def read_rcfile(path):
    """Reads a config file and returns all the entries in a hash.
    Use this for the activity rc files.
    The core libs have their own specialized rc readers.
    """
    config = ConfigParser.ConfigParser()
    if not os.path.exists(path):
        module_logger.info("No rc file found at: %s" % path)
        return {}
    try:
        config.read(path)
    except Exception, info:
        module_logger.error("Failed to parse rc file: %s" % info)
        return {}
    hash = {}
    for section in config.sections():
        hash[section] = dict(config.items(section))
    return hash

def read_unicode_file(path, lines=1):
    import codecs
    file = []
    try:
        f = codecs.open(path, 'r', encoding='utf-8')
        if lines:
            file = f.readlines()
        else:
            file = f.read()
    except Exception:
        module_logger.exception("Error reading: %s" % path)
        return None
    return file

def hex2ascii(hc):
    """Converts a hex string representing an char 0-9/a-z into a unicode string.
    Use this only for converting the alphabet soundfile names.
    'U0031' -> '1'""" 
    if hc[1] == 'U':
        name = hc[1:]
    else:
        name = hc
    i = int(hex(name), 16)
    text = unichr(i)
    return text

def ascii2hex(sc):
    """Converts a ascii string representing an char 0-9/a-z into a unicode hex string.
    Use this only for converting the alphabet soundfile names.
    u'1' -> 'U0031'"""
    if type(sc) is types.UnicodeType:
        sc = str(sc)
    hs = 'U%s' % (hex(ord(unicode(sc, 'utf8')))[2:].zfill(4))
    return hs

def map_keys(key_map, key):
    """map_keys --> mapped key when a map is specified else the key is returnt
    unchanged.
    @key_map is the keymap to use or None when no mapping is needed.
    @key is the key to map.
    Used to provide a keymapping for Hebrew, Arabic and Russian."""
    if not key_map:
        return key
    try:
        km = getattr(KeyMaps, key_map)
    except AttributeError:
        module_logger.error("No keymap: %s found" % key_map)
        return key
    else:
        if km.has_key(key):
            return km[key]
        else:
            module_logger.error("No key: %s found" % key)
            return key

class NoneSound:
    """Used by the load_sound and load_music functions to provide
     a bogus sound object.
     You can also test if your object is a NoneSound object or not.
     if sndobject == 'NoneSound' or if sndobject != 'NoneSound'."""
    def play(self, loop=None):
        pass
    def stop(self):
        pass
    def queue(self):
        pass
    def get_volume(self):
        return None
    def set_volume(self, sound_value):
        pass
    def __eq__(self, other):
        if other == 'NoneSound': return True
        else: return False
    def __ne__(self, other):
        if other != 'NoneSound': return True
        else: return False

class MusicObject:
    """This is a wrapper around pygame.music"""
    def __init__(self, filename):
        self.s = filename
    def __eq__(self, other):
        if other == 'MusicObject': return True
        else: return False
    def __ne__(self, other):
        if other != 'MusicObject': return True
        else: return False
    def play(self, loop=0):
        """loop defaults to 0. values > 0 to loop a number of times.
            -1 means loop forever."""
        pygame.mixer.music.load(self.s)
        pygame.mixer.music.play(loop)
    def stop(self):
        pygame.mixer.music.stop()
    def queue(self):
        pygame.mixer.music.queue(self.s)

def load_alphabetsound(char, loc='en'):
    """Loads an alphabet sound file for the @loc locale.
    If @loc is a 'complete' locale like nl_BE then first the nl_BE alphabetsound directory
    is look for. When that fails the 'nl' directory is queried and when that also
    fails the fallback 'en' is used.
    """
    name = ascii2hex(char)
    module_logger.debug("load_alphabetsound got locale %s" % loc)
    if not os.path.exists(os.path.join(ALPHABETDIR, loc)):
        module_logger.debug("load_alphabetsound cannot find %s" % os.path.join(ALPHABETDIR, loc))
        if '_' in loc:
            module_logger.debug("load_alphabetsound trying to use short name for locale")
            if os.path.exists(os.path.join(ALPHABETDIR, loc.split('_')[0])):
                loc = loc.split('_')[0]
            else:
                loc = 'en'
        else:
            loc = 'en'
    path = os.path.join(ALPHABETDIR, loc, name + '.ogg')
    module_logger.debug("load_alphabetsound trying to load %s" % path)
    if not os.path.exists(path):
        module_logger.debug("load_alphabetsound trying to use sound file name in upper case")
        path = os.path.join(ALPHABETDIR, loc, name.upper() + '.ogg')
    snd = load_sound(path)
    return snd
    
def speak_letter(letter, loc='en'):
    """Plays the alphabet soundfile for @letter.
    @loc is the locale.
    Return True on succes and False on failure"""
    try:
        load_alphabetsound(letter.lower(), loc).play()
    except Exception, info:
        module_logger.error("error while trying to play alphabet sounds: %s" % info)
        return False
    else:
        return True
            
def load_sound(name):
    """Loads a sound -> pygame sound object.
      If no file can be loaded return a dummy class"""
                
    if not pygame.mixer or not pygame.mixer.get_init():
        module_logger.info('Cannot load sound: %s, no mixer initialized' % name)
        module_logger.info('Using Nonesound')
        return NoneSound()
    if not os.path.exists(name):
        module_logger.info("Soundpath %s doesn't exists" % name)
        return NoneSound()
    try:
        sound = pygame.mixer.Sound(name)
    except pygame.error:
        module_logger.info('Cannot load sound: %s ' % name)
        module_logger.info('Using Nonesound')
        return NoneSound()
    return sound

def load_music(file):
    """  Set up music object, if the music can't be loaded a bogus object will be returnt.
     Beware that due to SDL limitations you can only have one music source loaded.
     This means that even when you have multiple pygame instances, there can only be one
     music source. This function returns a filename wrapped in a sound like object with
     a play and stop method.
     For multiple sources use the pygame.Sound and wave combination.
     """    
    if not pygame.mixer or not pygame.mixer.get_init():
        module_logger.info('Cannot load sound: %s, no mixer initialized' % file)
        return NoneSound()
    if not os.path.exists(file):
        module_logger.info('Cannot load sound: %s' % file)
        module_logger.info('Using Nonesound')
        playmusic = NoneSound()
    else:
        playmusic = MusicObject(file)

    return playmusic
    
def aspect_scale(img,(bx,by)):
    """ Scales 'img' to fit into box bx/by.
     This method will retain the original image's aspect ratio """
    ix,iy = img.get_size()
    if ix > iy:
        # fit to width
        scale_factor = bx/float(ix)
        sy = scale_factor * iy
        if sy > by:
            scale_factor = by/float(iy)
            sx = scale_factor * ix
            sy = by
        else:
            sx = bx
    else:
        # fit to height
        scale_factor = by/float(iy)
        sx = scale_factor * ix
        if sx > bx:
            scale_factor = bx/float(ix)
            sx = bx
            sy = scale_factor * iy
        else:
            sy = by
    return pygame.transform.scale(img, (int(sx),int(sy)))

def grayscale_image(surf):
    width, height = surf.get_size()
    for x in range(width):
        for y in range(height):
            red, green, blue, alpha = surf.get_at((x, y))
            L = 0.3 * red + 0.59 * green + 0.11 * blue
            gs_color = (L, L, L, alpha)
            surf.set_at((x, y), gs_color)
    return surf
    
def load_image(file, transparent=0, theme='childsplay'):
    """loads an image and convert it.
    When a theme is set it will try to load the image from the theme directory
    otherwise it will load the default image.
    Set transparent to 1 if you want to add transparenty to your non-alpha image.
    The color of the pixel located at 0,0 in your image will be taken as the trans layer.
    If you have a image with an alpha channel set transparent to 0 nd alpha to 1.
    ATTENTION, if you planning to rotate your image DON'T use alpha, it will crash pygame."""
    
    if theme and theme != 'childsplay':
        path, name = os.path.split(file)
        path = os.path.join(path, theme, name)
        try:
            surface = pygame.image.load(path)
        except pygame.error:
            module_logger.info('Failed to load theme image: %s' % path)
            module_logger.info('%s' % pygame.get_error())
            module_logger.info('Trying default images')
            theme = None
    if not theme or theme == 'childsplay':
        try:
            surface = pygame.image.load(file)
        except pygame.error:
            module_logger.exception('Could not load image "%s"\n %s' % (file, pygame.get_error()))
            MyError.line = 'Utils.py -> load_image()'
            raise MyError('Could not load image "%s"\n %s' % (file, pygame.get_error()))
    if transparent:
        surface.set_colorkey(surface.get_at((0, 0)), RLEACCEL)
    if surface.get_alpha():
        #if the image loaded has alpha, the overall alpha set here
        #(128) will be overriden. Do it anyway for forcing RLEACCEL
        surface.set_alpha(127, RLEACCEL)
        # if the display is in a lower mode, then surface.convert would
        # convert the surface in an alpha-less surface.
        return surface
    return surface.convert()

def char2surf(char, fsize, fcol=None, ttf='arial', bold=False, antialias=True, split=0):
    """Renders a character and returns the surface.
    char must be a string. Returns the surface.
    This now uses pango to render the fonts.
    @ttf must be a fontname installed on the system
    When you supply a name ending on '.ttf' a truetype font path is assumed and
    the pygame font renderer is used. 
    @split indicates the length of the strings. When it's set to 0, the default,
    it will not split the string and the surface will be returnt.
    When split > 0 a list with surfaces is returnt.
    """
    if NoGtk:
        ttf = TTF
    items = []
    if split:
        txt = txtfmt([char], split)
    else:
        txt = [char]
    if not fcol:
        fcol = (0, 0, 0)
    if ttf and os.path.splitext(ttf)[1] == '.ttf':
        #module_logger.debug('ttf=%s, using standard pygame font' % ttf)
        font = pygame.font.Font(ttf, fsize)
        for line in txt:
            if not txt: continue
            s = font.render(line, True, fcol)
            items.append(s)
    else:
        font = pangofont.PangoFont(family=ttf, size=fsize, bold=bold)
        font.set_bold(bold)
        for line in txt:
            s = font.render(line, True, fcol, None)
            items.append(s)
    if split:
        return items
    else:
        return items[0]
    
def text2surf(word, fsize, fcol=None, ttf=None, sizel=None, bold=False, antialias=True):
    """Renders a text in a surface and returns a surface,size list of the items
    sizelist is a list like this [(7,17),(10,17)] tuples are x,y size of the character."""
    if NoGtk:
        ttf = TTF
    if not fcol:
        fcol = (0, 0, 0)
    if ttf and os.path.splitext(ttf)[1] == '.ttf':
        module_logger.debug('ttf=%s, using standard pygame font' % ttf)
        font = pygame.font.Font(ttf, fsize)
        s = font.render(word, True, fcol)
    else:
        try:
            font = pangofont.PangoFont(family=ttf, size=fsize)
            font.set_bold(bold)
        except Exception, info:
            module_logger.error('%s. Using standard pygame font' % info)
            font = pygame.font.Font(None, fsize)
            s = font.render(word, True, fcol)
        else:
            try:
                s = font.render(word, True, fcol, None)# none means trasparent
            except Exception, info:
                module_logger.exception("Failed to render SDL font: %s" % info)
                return
    if sizel:
        sizelist = font.size(word)
    else:
        sizelist = map(font.size, word)
    return s, sizelist

def Ipl2NumPy(img):
    """ Converts an openCV cvArray to a pygame surface.
     Parameters:   IplImage img - image"""
    array = adaptors.Ipl2NumPy(img)
    surf = pygame.surfarray.make_surface(array)
    surf = pygame.transform.rotate(surf, -90)
    surf = pygame.transform.flip(surf, 1, 0)
    return surf

def NumPy2Ipl(surf):
    """Converts a pygame surface to an openCV cvArray.
    Parameters:   pygame.Surface surf - pygame Surface
    """
    py_image = surfarray.pixels3d(surf)
    cv_image = adaptors.NumPy2Ipl(py_image.transpose(1,0,2))
    return cv_image

def find_files(directory, patternlist):
    """Generator function to search files that match a certain pattern.
    patternlist must be a list of patterns.
    Usage: for file in find_files('topdir','*.foo'):
                print 'found:',file 
    """
    for root, dirs, files in os.walk(directory):
        for pattern in patternlist:
            for basename in files:
                if fnmatch.fnmatch(basename, pattern):
                    filename = os.path.join(root, basename)
                    yield filename

class Dimmer:
    """
Dimmer class

Tobias Thelen (tthelen@uni-osnabrueck.de)
6 September 2001

PUBLIC DOMAIN
Use it in any way you want...

tested with: Pyton 2.0/pygame-1.1, Windows 98

A class for 'dimming' (i.e. darkening) the entire screen, useful for:
- indicating a 'paused' state
- drawing user's attention away from background to e.g. a Quit/Don't Quit
  dialog or a highscore list or...

Usage:

dim=Dimmer(keepalive=1)
  Creates a new Dimmer object,
  if keepalive is true, the object uses the same surface over and over again,
  blocking some memory, but that makes multiple undim() calls possible - 
  Dimmer can be 'abused' as a memory for screen contents this way..

dim.dim(darken_factor=64, color_filter=(0,0,0))
  Saves the current screen for later restorage and lays a filter over it -
  the default color_filter value (black) darkens the screen by blitting a 
  black surface with alpha=darken_factor over it.
  By using a different color, special effects are possible,
  darken_factor=0 just stores the screen and leaves it unchanged

dim.undim()
  restores the screen as it was visible before the last dim() call.
  If the object has been initialised with keepalive=0, this only works once.

"""
    def __init__(self, keepalive=0):
        self.keepalive = keepalive
        if self.keepalive:
            self.buffer = pygame.Surface(pygame.display.get_surface().get_size())
        else:
            self.buffer = None
        
    def dim(self, darken_factor=64, color_filter=(0, 0, 0)):
        if not self.keepalive:
            self.buffer = pygame.Surface(pygame.display.get_surface().get_size())
        self.buffer.blit(pygame.display.get_surface(), (0, 0))
        if darken_factor > 0:
            darken = pygame.Surface(pygame.display.get_surface().get_size())
            darken.fill(color_filter)
            darken.set_alpha(darken_factor)
            # safe old clipping rectangle...
            old_clip = pygame.display.get_surface().get_clip()
            # ..blit over entire screen...
            pygame.display.get_surface().blit(darken, (0, 0))
            pygame.display.update()
            # ... and restore clipping
            pygame.display.get_surface().set_clip(old_clip)

    def undim(self):
        if self.buffer:
            pygame.display.get_surface().blit(self.buffer, (0, 0))
            pygame.display.update()
            if not self.keepalive:
                self.buffer = None
    
class MazeGen:
    """  Perfect maze generator, based on the Mazeworks algorithm.(adapted for Packid)
  Constuctor takes two uneven integers eg rows,cols. If the rows/cols are even
  then they will be decreased by one.
  Usage: m = MazeGen(17,17)
         maze = m.get_maze()
  maze is a tuple with tuples representing a grid where a 0 stands for a wall,
  and 1 for a room. The outer walls are also zeros.
  """
              
    def __init__(self, rows, cols):
        rows = rows-(rows % 2 == 0)
        cols = cols-(cols % 2 == 0)
        self.matrix = []
        for r in range(rows):
            self.matrix.append(([0] * cols))
        self._make_maze()
    
    def _make_maze(self):
        import random
        cellstack = []
        maxrow = len(self.matrix)
        row = random.choice(range(1, maxrow, 2))
        maxcol = len(self.matrix[0])
        col = random.choice(range(1, maxcol, 2))
        #print 'start row,col',row,col
        maxcol -= 3
        maxrow -= 3
        
        self.matrix[row][col] = 1
        
        nextcell = []
        while 1:
            nextcell = []
            #check neighbors
            if col < maxcol and self.matrix[row][col + 2] == 0:
                nextcell.append(((row, col + 1), (row, col + 2)))
            if col > 2 and self.matrix[row][col-2] == 0:
                nextcell.append(((row, col-1), (row, col-2)))
            if row < maxrow and self.matrix[row + 2][col] == 0:
                nextcell.append(((row + 1, col), (row + 2, col)))
            if row > 2 and self.matrix[row-2][col] == 0:
                nextcell.append(((row-1, col), (row-2, col)))
            
            if nextcell:
                next = random.choice(nextcell)
                # knock down the wall
                self.matrix[next[0][0]][next[0][1]] = 1
                self.matrix[next[1][0]][next[1][1]] = 1
                cellstack.append(((row, col)))# stack old cell
                row, col = next[1][0], next[1][1]
                
            else:# Backtrack our steps
                try:
                    row, col = cellstack.pop()
                except IndexError:
                    #print 'break'
                    break
        self.matrix[-2][-2] = 2
        self.matrix[-1][-2:] = [1, 1]
        
    def get_maze(self):
        grid = tuple(map(tuple, self.matrix))
        return grid

def import_module(filename, globals=None, locals=None, fromlist=None):
    """ Import any module without changes to sys.path.
      Taken from the library reference.(Never invent the wheel twice)"""
    # Fast path: see if the module has already been imported.
    try:
        return sys.modules[filename]
    except KeyError:
        pass
    path, name = os.path.split(filename)
    name, ext = os.path.splitext(name)
    fp = None    
    try:
        fp, pathname, description = imp.find_module(name, [path])
        return imp.load_module(name, fp, pathname, description)
        if fp: fp.close()
    except (StandardError, MyError), info:
        module_logger.exception("Import of %s failed" % filename)
        raise MyError, info
        if fp: fp.close()
    
def txtfmt(text, split):
    """ Formats a list of strings in a list of strings of 'split' length.
       returns a new list of strings.
       text = list of strings
       split = integer
       """
    return textwrap.wrap('\n'.join(text), split)

def txtfmtlines(text, split):
    """Formats a list of strings of 'split' length.
    Returns a list of strings with single linebreaks preserved.
    This differs from txtfmt which removed single line linebreaks
    """
    wrapper = textwrap.TextWrapper()
    wrapper.width = split
    wrapper.drop_whitespace = False
    lines = []
    for line in text:
        if not line.split('\n'):
            lines += line
        else:
            lines += wrapper.wrap(line)
    return lines

class ScaleImages:
    def __init__(self, imgObjects, TargetSize, stdCardObj=None):
        """ ImgObjects = sequence of SDL objects, can also be a dictionairy but in that
                    case you can't use stdCard. If you set it to None you MUST pass a
                    surface to the get_images method.
                    This way you can use it to construct a scaler and each time pass
                   a surface to get_images which will be blitted on the stdCardObj. 
        stdCardObj = None means no blitting on card, just return
                   the scaled images. Else the images are blitted on this object.
        TargetSize = desired scale size (tuple)
        """
        self.logger = logging.getLogger("childsplay.utils.ScaleImages")
        self.TargetSize = pygame.Rect((0, 0) + TargetSize)# convert this tuple in a pygame rect
            # see: def _scale_if_needed
        if stdCardObj:
            # in case of error, we don't use a card back
            # just return scaled images.
            border = 10 # border size of card (memory), 2 * border size
            try: 
                self.stdCard = stdCardObj
                # scale the card also to TargetSize, so we use one card for everything
                self.scaled_card = self._scale_if_needed(self.stdCard)
                self.TargetSize.inflate_ip(-border, -border)# reduce the size of the rect, so the images are smaller
                    # then the card (compensate for border)
            except StandardError, info:
                self.logger.exception("Failed to scale stdCardObj:%s, %s" % (stdCardObj, info))
                self.stdCard = None
        else:
            self.stdCard = None #to use as a test later, if we gonna blit the img on a card or not
                # see get_images
        self.imgObjects = imgObjects
        
    def get_images(self, image=None):
        """ This returns a list with scaled images, blitted on a card if
         it was parsed to the class constructor.
         When image is a surface image is scaled on a card and returnt in list of one
        """
        if type(self.imgObjects) == types.DictType:
            # This won't work in combination with stdCard
            imgs = {}
            for k, v in self.imgObjects.items():
                imgs[k] = self._scale_if_needed(v)
            return imgs
        if image:
            self.imgObjects = (image, )
        imgs = map(self._scale_if_needed, self.imgObjects) #this returns always a surface, scaled or not
        if self.stdCard:# we have a blanc card to blit the images on
            card_imgs = []
            for img in imgs:
                if self.scaled_card.get_alpha():
                    card = self.scaled_card.convert_alpha()
                else:
                    card = self.scaled_card.convert()
                # center the image on the card
                center_pos = img.get_rect()
                center_pos.center = self.TargetSize.center # Wow, magic, see the pygame reference on Rect
                    # a good understanding of the pygame.Rect is a real time saver.
                card.blit(img, center_pos)
                card_imgs.append((card))
            imgs = card_imgs[:]# a real copy, just to be sure :-)
        return imgs   
        
    def _scale_if_needed(self, img):
        """ We only scale down, not up because the result of upscaling sucks"""
            # Remember, TargetSize is a pygame rect (contains is a method of Rect)
        if not self.TargetSize.contains(pygame.Rect((0, 0) + img.get_size())):
            return self._scale_card(img)
        else:
            return img  #not changed!
            
    def _scale_card(self, img):
        """ This does the actual scaling and returns a scaled SDL surface."""
        imgSize = img.get_size()
        
        # we assume TargetSize = x == y, a square
        # TODO XXX what if Target not a square ??
        if (imgSize[0] > imgSize[1]):# which one should we divide to get the longest side within the TargetSize 
            scale_ratio = float(imgSize[0]) / self.TargetSize[2]# TargetSize is a rect -> (x,y,x-size,y-size)
        else:
            scale_ratio = float(imgSize[1]) / self.TargetSize[3]
                           
        scale_x = int(imgSize[0] / scale_ratio)
        scale_y = int(imgSize[1] / scale_ratio)
        
        scaled_img = pygame.transform.scale(img, (scale_x, scale_y))
        return scaled_img

# TODO: Do we want to use GfxCursor?
class GfxCursor:
    """
    Replaces the normal pygame cursor with any bitmap cursor.
    This is a nice little GfxCursor class that gives you arbitrary mousecursor
    loadable from all SDL_image supported filetypes. 

    Author: Raiser, Frank aka CrashChaos (crashchaos at gmx.net)
    Author: Shinners, Pete aka ShredWheat
    Version: 2001-12-15

    Usage:
    Instantiate the GfxCursor class. Either pass the correct parameters to
    the constructor or use setCursor, setHotspot and enable lateron.

    The blitting is pretty optimized, the testing code at the bottom of
    this script does a pretty thorough test of all the drawing cases.
    It enables and disables the cursor, as well as uses a changing background.

    In your mainloop, the cursor.show() should be what you draw last
    (unless you want objects on top of the cursor?). Then before drawing
    anything, be sure to call the hide(). You can likely call hide() immediately
    after the display.flip() or display.update().

    The show() method also returns a list of rectangles of what needs to be
    updated. You can also move the cursor with pygame.mouse.set_pos()


    That's it. Have fun with your new funky cursors.
    

    """

    def __init__(self, surface, cursor=None, hotspot=(0, 0)):
        """
        surface = Global surface to draw on
        cursor  = surface of cursor (needs to be specified when enabled!)
        hotspot = the hotspot for your cursor
        """
        self.surface = surface
        self.enabled = 0
        self.cursor  = None
        self.hotspot = hotspot
        self.bg      = None
        self.offset  = 0, 0
        self.old_pos = 0, 0
        
        if cursor:
            self.setCursor(cursor, hotspot)
            self.enable()

    def enable(self):
        """
        Enable the GfxCursor (disable normal pygame cursor)
        """
        if not self.cursor or self.enabled: return
        pygame.mouse.set_visible(0)
        self.enabled = 1

    def disable(self):
        """
        Disable the GfxCursor (enable normal pygame cursor)
        """
        if self.enabled:
            self.hide()
            pygame.mouse.set_visible(1)
            self.enabled = 0

    def setCursor(self, cursor, hotspot=(0, 0)):
        """
        Set a new cursor surface
        """
        if not cursor: return
        self.cursor = cursor
        self.hide()
        self.show()
        self.offset = 0, 0
        self.bg = pygame.Surface(self.cursor.get_size())
        pos = self.old_pos[0]-self.offset[0], self.old_pos[1]-self.offset[1]
        self.bg.blit(self.surface, (0, 0),
            (pos[0], pos[1], self.cursor.get_width(), self.cursor.get_height()))

        self.offset = hotspot

    def setHotspot(self, pos):
        """
        Set a new hotspot for the cursor
        """
        self.hide()
        self.offset = pos

    def hide(self):
        """
        Hide the cursor (useful for redraws)
        """
        if self.bg and self.enabled:
            return self.surface.blit(self.bg,
                (self.old_pos[0]-self.offset[0], self.old_pos[1]-self.offset[1]))

    def show(self):
        """
        Show the cursor again
        """
        if self.bg and self.enabled:
            pos = self.old_pos[0]-self.offset[0], self.old_pos[1]-self.offset[1]
            self.bg.blit(self.surface, (0, 0),
                (pos[0], pos[1], self.cursor.get_width(), self.cursor.get_height()))
            return self.surface.blit(self.cursor, pos)

    def update(self, event):
        """
        Update the cursor with a MOUSEMOTION event
        """
        self.old_pos = event.pos


        
def current_time():
    """Maincore uses this to get the current time to set the 'time_start'
    and 'time_end' values in the dbase table"""
    t = time.strftime("%y-%m-%d_%H:%M:%S", time.localtime())
    module_logger.debug("current_time returns %s" % t)
    return t
def calculate_time(start, end):
    """The 'end'time is substracted from 'start'time and the result time is
    returned in a format suitable to put into the dbase.
    Times must be in the format as returnt from current_time.
    """
    logger = logging.getLogger("childsplay.utils.calculate_time")
    logger.debug("Called with start:%s, end:%s" % (start, end))
    start = start.replace('-', ':').replace('_', ':')
    end = end.replace('-', ':').replace('_', ':')
    arg_key0 = [int(s) for s in start.split(':')]
    arg_key1 = [int(s) for s in end.split(':')]
    #t1 = datetime.datetime.now()
    #t2 = datetime.datetime.now()
    #t3 = t2-t1
    #t3.seconds / 60
    #t3.seconds % 60
    dt0 = datetime.datetime( * arg_key0)
    dt1 = datetime.datetime( * arg_key1)
    dt2 = dt1 - dt0
    h, m = dt2.seconds / 60, dt2.seconds % 60
    return "%02d:%02d" % (h, m)
def sleep(ms):
    """Pause the process for the given milliseconds and clears the event queue when
    done."""
    pygame.time.delay(ms)
    pygame.event.clear()

def shadefade(textstring, textsize, amount,bold=False, \
              fcol=(200, 200, 200), shadecol=(255, 255, 255)):
    """Render @textstring onto a surface with a 3d dropshadow-like effect.
    Returns a SDL surface.
    Originally written by Brendan Becker, http://clickass.org/
    """
    if PLATFORM == 'win32':
        textsize -= 16
    
    font = pangofont.PangoFont(None, textsize, bold=bold)
    # determine the size of the text surface so that we can center it later on
    basetext = font.render(unicode(textstring), 1, fcol, None)
    width, height = basetext.get_size()
    displaysurface = pygame.Surface((width, height), SRCALPHA, 32)
    tempsurface = pygame.Surface((width + amount, height+amount), SRCALPHA, 32)
    
    for i in range(amount):
        camt = amount-i
        r = shadecol[0] / camt
        g = shadecol[1] / camt
        b = shadecol[2] / camt
        if r < 0:  r = 0
        if g < 0:  g = 0
        if b < 0:  b = 0
        text = font.render(unicode(textstring), 1, (r, g, b), None)
        tempsurface.blit(text, (camt, camt))
    tempsurface.blit(basetext, (0, 0))
    displaysurface.blit(tempsurface, (0, 0))
    return displaysurface

def _set_lock(timeout=10):
    """Set a lock file to prevent starting multiple instances of childsplay_sp.
    It will first look if a lock has been set previously, if so it will check the
    'epoch time' and when it's @timeout seconds in the past it will remove the lock
    and set a new one.
    The timeout is done to prevent stale locks that will prevent restarting childsplay
    in case of a crash.
    The maincore will also register a _remove_lock function at the atexit class."""
    if os.path.exists(LOCKFILE):
        try:
            f = open(LOCKFILE, 'r')
            lines = f.readlines()
            t = float(lines[0][:-1])
        except (IOError, TypeError):
            module_logger.exception("failed to read master lockfile")
            _remove_lock()
            return True
        if time.time() - t < timeout:
            module_logger.info("Lock timeout not yet reached")
            return False
        else:
            _remove_lock()
            return True
    module_logger.info("Setting master lock: %s" % LOCKFILE)
    t = str(time.time())
    try:
        f = open(LOCKFILE, 'w')
        f.write(t)
        f.close()
    except IOError:
        module_logger.exception("failed to set master lockfile")
        _remove_lock()
        return False
    else:
        module_logger.debug("Lock set to %s" % t)
        f.close()
        return True
    
def _remove_lock():
    """Used to cleanup any locks"""
    module_logger.info("Removing master lock")
    if os.path.exists(LOCKFILE):
        try:
            os.remove(LOCKFILE)
        except IOError:
            module_logger.exception("failed to remove master lockfile")
            module_logger.error("Please remove lock file: % manually" % LOCKFILE)
    
def score2percentile(s_list, mean, sd):
    """Returns the percentile in a list with the coresponding dates
    calculated from the given parameters.
    @s_list is a list with tuples: [(date,score),(date,score)....]
    @mean is an float holding the mean from the dbase
    @sd is an float holding the sd from the dbase
    
    The z - Score is simply the number of Standard Deviation units a student's
    raw score is above or below the mean.  It has a one-to-one relationship with
    the standard deviation unit; 
    one z-score unit = one standard deviation unit and a z-score of zero is the mean.
    Further discussion of the standard deviation unit as well as it's use in 
    analyzing student scores is beyond the scope of this document.
    The z - Score is arrived at by taking the raw score for a student,  
    subtracting the mean (average) of all student scores and dividing by 
    the standard deviation of the student raw scores.
    
    zS = (RS - Mean) / SD
    Where:
    zS is the z - Score for the Student
    RS is the student's raw score
    Mean is the mean (average) of the student raw scores
    SD is the Standard Deviation of the student raw scores.
    
    Then we find the percentile in a hash table and multiply it by 10 to get a 
    nice value to display.
    """
    module_logger.debug("score2percentile called with:%s,%s,%s" % (s_list, mean, sd))
    # we unpack the score/date list, make the calculation and build
    # a new score/date list
    new_s_list = []
    for date, rs in s_list:
        if not rs:
           continue
        zs = (float(rs) - float(mean)) / float(sd)
        if zs < 0:
            prc = 1.0 - float(normal_distribution_table["%.2f" % abs(zs)])
            prc = str(prc *10)
        else:
            prc = str(float(normal_distribution_table["%.2f" % zs]) * 10)[:4]
        module_logger.debug("rs: %s, mean:%s, sd:%s, zscore:%s, percentile:%s" % (rs, mean, sd, zs, prc))
        new_s_list.append((date, prc))
    return new_s_list
    
class OrderedDict(dict, MutableMapping):
    # Taken from Python 2.7, runs also on python  >= 2.4.
    # Methods with direct access to underlying attributes
    def __init__(self, *args, **kwds):
        if len(args) > 1:
            raise TypeError('expected at 1 argument, got %d', len(args))
        if not hasattr(self, '_keys'):
            self._keys = []
        self.update(*args, **kwds)

    def clear(self):
        del self._keys[:]
        dict.clear(self)

    def __setitem__(self, key, value):
        if key not in self:
            self._keys.append(key)
        dict.__setitem__(self, key, value)

    def __delitem__(self, key):
        dict.__delitem__(self, key)
        self._keys.remove(key)

    def __iter__(self):
        return iter(self._keys)

    def __reversed__(self):
        return reversed(self._keys)

    def popitem(self):
        if not self:
            raise KeyError
        key = self._keys.pop()
        value = dict.pop(self, key)
        return key, value

    def __reduce__(self):
        items = [[k, self[k]] for k in self]
        inst_dict = vars(self).copy()
        inst_dict.pop('_keys', None)
        return (self.__class__, (items,), inst_dict)

    # Methods with indirect access via the above methods

    setdefault = MutableMapping.setdefault
    update = MutableMapping.update
    pop = MutableMapping.pop
    keys = MutableMapping.keys
    values = MutableMapping.values
    items = MutableMapping.items

    def __repr__(self):
        pairs = ', '.join(map('%r: %r'.__mod__, self.items()))
        return '%s({%s})' % (self.__class__.__name__, pairs)

    def copy(self):
        return self.__class__(self)

    @classmethod
    def fromkeys(cls, iterable, value=None):
        d = cls()
        for key in iterable:
            d[key] = value
        return d

#try:
#    import db_conf as dbrc
#    rckind = 'db_conf'
#except ImportError:
#    import db_dev as dbrc
#    rckind = 'db_dev'
#rc_hash = dbrc.rc
#if rc_hash['default']['production']:
#    kind = 'production'
#else:
#    kind = 'develop'
#rc_hash['kind'] = kind
#try:
#    WEAREPRODUCTION = int(rc_hash['default']['production'])
#except KeyError:
#    WEAREPRODUCTION = '0'
WEAREPRODUCTION = '0'

    



# -*- coding: utf-8 -*-

# Copyright (c) 2006 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           SPgdm.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# Graphical login manager, like the gdm.
# The difference is that this is started by schoolsplay DataManager much like a splash screen.
# There will be an option for SP to disable this and run in a anonymous version.

# Start SP like this to disable the login stuff: schoolsplay --anonymous

import logging
import os
module_logger = logging.getLogger("childsplay.SPgdm")

from SPConstants import *
from SPSpriteUtils import SPInit

from utils import char2surf, load_image

CORE_BUTTONS_XCOORDS = range(6, 790, 88)

import pygame

from pygame.constants import *

#import childsplay_sp.ocempgui.widgets as ocw
#import childsplay_sp.ocempgui.widgets.Constants as ocwc

if __name__ == '__main__':
    # needed to simulate gettext
    import __builtin__
    __builtin__.__dict__['_'] = lambda x:x
        
#from SPocwWidgets import InfoDialog, ExitDialog, SPEntry, SPLabel
#from SPVirtualkeyboard import VTKEscapeKeyException
import SPHelpText
import Version

# Load the ocempgui theme.
# TODO" make ocw theme configurable
# TODO: this is not working
#ocw.base.GlobalStyle.load (os.path.join(ACTIVITYDATADIR,'ocw','themes','schoolsplay.rc'))

class GDMEscapeKeyException(Exception):
    """ This is raised from the activity_loop when the user hits escape.
    We basically using this exception as a signal"""
    pass
    
class SPGreeter:
    """Starts a login screen, this will be a window, not fullscreen.
    """
    def __init__(self, cmd_options, theme='default', vtkb=None, fullscr=None):
        global ACTIVITYDATADIR
        self.logger = logging.getLogger("childsplay.SPgdm.SPGreeter")
        self.logger.debug("Starting")
        self.cmd_options = cmd_options
        self.__name = ''
        captxt = _("Childsplay_sp login")
        if self.cmd_options.theme != 'default':
            captxt = captxt.replace('Childsplay_sp', self.cmd_options.theme)
        ICONPATH = os.path.join(ACTIVITYDATADIR, 'SPData', 'themes', self.cmd_options.theme)
        DEFAULTICONPATH = os.path.join(ACTIVITYDATADIR, 'SPData', 'themes', 'default')
        self.vtkb = vtkb# is used in the _run_loop
        # setup screen
        size = (800, 600)
        if fullscr:
            self.screen = pygame.display.set_mode(size, FULLSCREEN)
        else:
            self.screen = pygame.display.set_mode(size)
        p = os.path.join(ICONPATH, 'spgdm_back.png')
        if not os.path.exists(p):
            p = os.path.join(DEFAULTICONPATH, 'spgdm_back.png')
        back = load_image(p)
        self.screen.blit(back, (0, 0))
        # get the version image
        vs = "Childsplay_sp version: %s" % Version.version
        if self.cmd_options.theme != 'default':
            vs = vs.replace('Childsplay_sp', self.cmd_options.theme)
        if self.cmd_options.adminmode:
            vs = vs + " (Adminmode)"
        vsurf = char2surf(vs, TTFSIZE-4, (0, 0, 0), ttf=TTF, bold=True, antialias=False)
        self.screen.blit(vsurf, (300, 0))
        
        self.actives = SPInit(self.screen, self.screen.convert())
        
        pygame.display.set_caption(captxt.encode('utf-8'))

        # setup our SP widgets
        label = SPLabel(_("Username:"), fontsize=TTFSIZE + 2)
        label.moveto((340, 250))
        self.actives.add(label)
        
        self.entry = SPEntry((340, 280), 9)
        self.entry.display_sprite()
        self.actives.add(self.entry)
        
        # setup ocempgui widgets
        self.renderer = ocw.Renderer()
        self.renderer.set_screen(self.screen)
        p = os.path.join(ICONPATH, 'spgdm_login_button.png')
        if not os.path.exists(p):
            p = os.path.join(DEFAULTICONPATH, 'spgdm_login_button.png')
        but = ocw.ImageButton(p)
        # The button resize accoording to the string size, this sucks.
        # We now must check the tring length to prevent the button from overflowing :-)
        # Looks like the C days are here again :-D
        t = _("Login")
        but.set_text(t)# we set the fontsize below
        
        but.child.create_style()
        but.child.style["font"]["size"] = TTFSIZE
        but.child.style['bgcolor'][ocwc.STATE_NORMAL] = KOBALT_LIGHT_BLUE
        but.create_style()['bgcolor'][ocwc.STATE_NORMAL] = KOBALT_LIGHT_BLUE
        but.connect_signal(ocwc.SIG_CLICKED, self._login_button_callback, self.entry)
        #but.opacity = 255
        but.topleft = (340, 320)
        # We clear the event queue as we sometimes get an crash with the message
        # error: Event queue full
        # Nothing to be found on Google so I've put a clear before the error
        # was generated.
        pygame.event.clear()
        self.renderer.add_widget(but)
        
        # logout button
        p = os.path.join(ICONPATH, 'spgdm_quit_button.png')
        if not os.path.exists(p):
            p = os.path.join(DEFAULTICONPATH, 'spgdm_quit_button.png')
        but = ocw.ImageButton(p)
        but.create_style()['bgcolor'][ocwc.STATE_NORMAL] = KOBALT_LIGHT_BLUE
        but.connect_signal(ocwc.SIG_CLICKED, self._quit_button_callback, but)
        #but.opacity = 180
        but.topleft = (720, 520)
        self.renderer.add_widget(but)
        
        # info button
        p = os.path.join(ICONPATH, 'spgdm_info_button.png')
        if not os.path.exists(p):
            p = os.path.join(DEFAULTICONPATH, 'spgdm_info_button.png')
        but = ocw.ImageButton(p)
        #but.set_text(_("Quit"))
        #but.child.create_style()["font"]["size"] = 36
        but.create_style()['bgcolor'][ocwc.STATE_NORMAL] = KOBALT_LIGHT_BLUE
        but.connect_signal(ocwc.SIG_CLICKED, self._info_button_callback, but)
        #but.opacity = 180
        but.topleft = (20, 520)
        self.renderer.add_widget(but)
        
        self.clock = pygame.time.Clock()
        self.runloop = True
        
        pygame.display.update()
        # run the virtual keyboard if we have one.
        if self.vtkb:
            self._run_vtkb_loop(self.renderer)
        else:
            # else we run a 'normal' loop.
            self._run_loop()
        
    def _run_loop(self):
        #self.runloop=0 # Used when profiling this module
        while self.runloop:
            self.clock.tick(30)
            pygame.event.pump()
            events = pygame.event.get()
            for event in events:
                if event.type is KEYDOWN and event.key is K_ESCAPE or event.type is QUIT:
                    self.run_event_loop = False
                    self._quit_button_callback(None)
                elif event.type is KEYDOWN:
                    if self.actives.refresh(event):
                        self._login_button_callback(self.entry)
            self.renderer.distribute_events( * events)
            
            
    def _run_vtkb_loop(self, parent_re=None):
        self.vtkb.show()
        kb = self.vtkb.echo_run(parent_re=[parent_re, self.renderer])
        word = []
        while self.runloop:
            try:
                k = kb.next()# this will return a string or none
                if k and k not in ('quit', 'enter'):# user hits anything but None, enter or escape
                    word.append(k)
                    self.entry.set_text(''.join(word))
                elif k == 'quit':# user hits escape
                    self._quit_button_callback()
                    break
                elif k == 'enter':
                    break
            except (StopIteration, VTKEscapeKeyException):
                self._quit_button_callback()
                break
                
    def _login_button_callback(self, entry):
        self.logger.debug("_login_button_callback called with %s" % entry)
        self.__name = entry.text
        self.runloop = False

    def _quit_button_callback(self,  * args):
        self.logger.debug("_quit_button_callback called")
        if not self.cmd_options.noexitquestion:
            dlg = ExitDialog(self.renderer)
            c = dlg.run()
        else:
            c = 0
        if c == 0:
            self.logger.info("User wants exit")
            raise GDMEscapeKeyException # let schoolsplay.py decide what next

    def _info_button_callback(self, but):
        self.logger.debug("_info_button_callback called")
        d = InfoDialog(self.renderer, SPHelpText.SPgdm._info_button_callback)
        d.run()

    def get_loginname(self):
        return self.__name
        
if __name__ == '__main__':
    def main():
        class Fake:
            adminmode = False
            theme = 'default'
            noexitquestion = None
        try:
            g = SPGreeter(Fake())
        except GDMEscapeKeyException:
            pass
        else:
            print "got name: %s" % g.get_loginname()

    import cProfile
    prof = cProfile.run('main()','profiler_out_0')
 
    
        





# AUTO-GENERATED MODULE, DON'T EDIT
# This module holds all the paths needed for childsplay_sp.
import os, sys

try:
    import IamBinary
except ImportError:
    print "No binary"
    BASEDIR = sys.path[0]
    os.chdir(BASEDIR)

DOCDIR = 'doc/childsplay_sp'
BASEDIR = os.getcwd()
SHARELIBDATADIR = os.path.join(BASEDIR, 'lib')
ALPHABETDIR = os.path.join(BASEDIR,'alphabetsounds')
LOCALEDIR = os.path.join(BASEDIR, 'locale')
WWWDIR = 'www/backend'



# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zytkiewicz stas.zytkiewicz@schoolsplay.org
#
#           spinbottle.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation. A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#create logger, logger was setup in SPLogging
import logging
# In your Activity class -> 
# self.logger =  logging.getLogger("childsplay.completion.Activity")
# self.logger.error("I don't understand logger")
# See SP manual for more info 

module_logger = logging.getLogger("childsplay.spinbottle")

# standard modules you probably need
import os,sys, random, textwrap, types, glob
import fnmatch
import pygame
from pygame.constants import *

import utils
from SPConstants import *
import SPSpriteUtils
from SPVirtkeyboard import KeyBoard, WordCompleter

#sys.path.insert(0, './lib')
import synonyms

    
class SpinningWheel(pygame.sprite.Sprite):
    def __init__(self, img, initial_position):
        pygame.sprite.Sprite.__init__(self)
        self.loaded_image = img
        self.image=self.loaded_image
        self.rotate=0 
        self.angle=15 
        self.rect = self.image.get_rect()
        self.rect.topleft = initial_position
        self.next_update_time = 0 # update() hasn't been called yet.
        self.selected=0

    def clicked(self, target):
        # FIXME: Dead code ??
        "returns true if the iwheel gets clicked"
#        if not self.selected:
#            self.selected= 1
        hitbox = self.rect.inflate(-5, -5)
        #print "Wheel got clicked"
        return hitbox.colliderect(target)

    def update(self, current_time, bottom):
        # FIXME: What's the purpose ? It doesn't stop at the letter
        # Update every 10 milliseconds = 1/100th of a second.
        if self.next_update_time < current_time:
            # rotate and recenter sprite
            oldCenter=self.rect.center
            self.rotate += self.angle
            self.image = pygame.transform.rotate(self.loaded_image, self.rotate)
            self.rect = self.image.get_rect()
            self.rect.center=oldCenter
            self.next_update_time = current_time + 20 

class Activity(synonyms.Activity):
    """  Base class mandatory for any SP activty.
    The activity is started by instancing this class by the core.
    This class must at least provide the following methods.
    start (self) called by the core before calling 'next_level'.
    post_next_level (self) called once by the core after 'next_level' *and* after the 321 count. 
    next_level (self,level) called by the core when the user changes levels.
    loop (self,events) called 40 times a second by the core and should be your 
                      main eventloop.
    get_helptitle (self) must return the title of the game can be localized.
    get_help (self) must return a list of strings describing the activty.
    get_helptip (self) must return a list of strings or an empty list
    get_name (self) must provide the activty name in english, not localized in lowercase.
    stop_timer (self) must stop any timers if any.
  """

    def __init__(self,SPGoodies):
        """SPGoodies is a class object that SP sets up and will contain references
        to objects, callback methods and observers
        TODO: add more explaination"""
        self.logger =  logging.getLogger("childsplay.spinbottle.Activity")
        self.logger.info("Activity started")
        synonyms.Activity.__init__(self, SPGoodies)
        
    def setup(self):
        # The location of the activities Data dir
        self.my_datadir = os.path.join(self.SPG.get_libdir_path(),'CPData','SpinbottleData')
        self.rchash = utils.read_rcfile(os.path.join(self.my_datadir, 'spinbottle.rc'))
        self.currentletter = None
        self.wordlisthash = {}
        paths = glob.glob(os.path.join(self.my_datadir, self.theme,'data_%s' % self.language, '*.txt'))
        self.logger.debug("found: %s " % paths)
        for p in paths:
            f = open(p, 'r')
            lines = f.readlines()
            f.close()
            self.wordlisthash[lines[0][:-1]] = lines[1:]
        self.categories = self.wordlisthash.keys()
        self.logger.debug("Found categories %s" % self.categories)
        random.shuffle(self.categories)
        self.currentcategory = self.categories.pop()
        self.questiontext = _("beginning with the letter: ")
        self.foundtext =  _("Already found:")
        self.suggestiontext = _("Suggestions:")
        self.wc = WordCompleter([])
        self.newexercise_counter = 3
    
    def _on_newquestionbut_clicked(self, *args):
        self.logger.debug("_on_newquestionbut_clicked called")
        self.newexercise_counter -= 1
        self.found_x, self.found_y = self.textright_pos[0], self.textright_pos[1] + 30
        self.kb.clear_display()
        txt = _("Starting a new exercise")
        self.next_exercise() 
 
    def next_exercise(self, currentletter=None):
        self.logger.debug("next_exercise called with %s" % currentletter)
        self.found_x, self.found_y = self.textright_pos[0], self.textright_pos[1] + 75
        if len(self.categories) < 1:
                self.categories = self.wordlisthash.keys()
                random.shuffle(self.categories)
        # Change self.newexercise_counter < 4 into 3 to get three times the same category wit a different letter. 
        if not self.currentcategory or self.newexercise_counter < 4:
            self.currentcategory = self.categories.pop()
            self.newexercise_counter = 3
        
        sequence = []
        for line in self.wordlisthash[self.currentcategory][2:]:
            c = line[1].lower()
            if c not in sequence:
                sequence.append(c)
            
        random.shuffle(sequence)
        if not currentletter:
            self.currentletter = sequence.pop()
        self.logger.debug("picked first letter %s" % self.currentletter)
        
        self.screen.blit(self.background, (self.x, self.y))
        self.backgr.blit(self.background, (self.x, self.y))
        
        self.wc.set_firstletter(self.currentletter)
        self.kb._set_firstletter(self.currentletter, self.wrong_sound)
        wordlist = self.wordlisthash[self.currentcategory][1:]
        self.wc.set_wordlist(wordlist)
        self.kb.show()
        
        h = 0
        txt = self.wordlisthash[self.currentcategory][:1][0][:-1]
        s = utils.char2surf(txt, fsize=20, fcol=self.fgcol)
        r = self.screen.blit(s,(self.texttop_pos[0], self.texttop_pos[1]+h))
        h += s.get_rect().h
        s = utils.char2surf(self.questiontext, fsize=20, fcol=self.fgcol)
        r = self.screen.blit(s,(self.texttop_pos[0], self.texttop_pos[1]+h))
        
        ls_pos = (r.topright[0], r.topright[1]-4)
        
        s = utils.char2surf(self.foundtext, fsize=20, fcol=self.fgcol)
        self.screen.blit(s, self.textright_pos)
        pygame.display.update()
        #### Start animation, increase or decrease attribute value from pygame.time.wait
        for i in range(4, 32):
            s = utils.char2surf(self.currentletter.upper(), fsize=i, fcol=RED, bold=True)
            r = self.screen.blit(s, ls_pos)
            pygame.display.update(r)
            pygame.time.wait(10)
            pygame.display.update(self.screen.blit(self.backgr, r, r))
        s = utils.char2surf(self.currentletter.upper(), fsize=i, fcol=self.fgcol, bold=True)
        r = self.screen.blit(s, ls_pos)
        pygame.display.update(r)
        ########################################################################
        self.actives.add(self.newquestionbut)
        self.newquestionbut.display_sprite()
        
        self.wc_x,  self.wc_y = self.wc_pos
        
        pygame.display.update()

    def get_helptitle(self):
        """Mandatory method"""
        return _("Spin the bottle")
    
    def get_name(self):
        """Mandatory method, returnt string must be in lowercase."""
        return "spinbottle"
    
    def get_help(self):
        """Mandatory methods"""
        text = [_("The aim of this activity:"),\
                " ", \
                _("On the next screen you'll find a keyboard, try to name as much %s as possible with the same letter.") % (self.currentcategory), \
                _("Only words from the dictionary are allowed."), \
               _("Hit the 'Begin' button to start." ), \
               " ", \
                _("Number of levels : 6")]
        
        return text 
    
    def get_helptip(self):
        """Mandatory method, when no tips available returns an empty list"""
        return []
        
    def get_helptype(self):
        """Mandatory method, you must set an type"""
        # Possible types are: Memory, Math, Puzzle, Keyboardtraining, Mousetraining
        #                     Language, Alphabet, Fun, Miscellaneous
        return _("Miscellaneous")
        
    
    def get_helplevels(self):
        """Mandatory method, must return a string with the number of levels
        in the follwing format:
        _("This level has %s levels") % number-of-levels"""
        return _("This activity has %s levels") % 6
    
    def get_score(self,timespend):
        """Mandatory method.
        @timespend is the time spend inside the level as it's calculated
        by the core.
        returns the score value for the past level.
        return None if no score value is used"""
        m,s = timespend.split(':')
        seconds = int(m)*60 + int(s)
    
    
        



# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           SPMoviePlayer.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import pygame
from pygame.constants import *
from SPWidgets import Button, Label
import logging
from SPColors import *
from SPSpriteUtils import SPSprite

class MoviePlayer:
    def __init__(self, file, actives, rect, observer, hidebuttons=False):
        """rect must be a pygame.Rect indicating the position and screen size the player
        can use for the movie and buttons.
        TODO: add more info"""
        self.logger = logging.getLogger('schoolsplay.SPMoviePlayer.MoviePlayer')
        self.maxW = rect.width
        butH = 50
        self.maxH = rect.height - butH # leave room for buttons
        self.observer = observer
        self.posX,  self.posY = rect.topleft
        self.actives = actives
        self.file = file 
        
        self._movie = pygame.movie.Movie(file)
        w, h = self._movie.get_size()
        w = min(int(w * 1.3 + 0.5), self.maxW)
        h = min(int(h * 1.3 + 0.5), self.maxH)
        self._surf = pygame.Surface((w, h))
        pygame.draw.rect(self._surf, GREY, self._surf.get_rect(), 4)
        self._movie.set_display(self._surf, pygame.Rect(4, 4, w-8, h-8))
        
        self.butlist = []       
        # TODO: these coords are hardcoded and depend on a certain surf size.
        # make it dynamic
        y = self.posY + self.maxH +10
        butXpos = (self.posX + 60, self.posX + 170, self.posX + 340, self.posX + 510)
        
        # TODO: We should use images iso text
        but = Button('Quit', (butXpos[0], y), padding=6)
        but.connect_callback(self._on_quit_but_cbf, MOUSEBUTTONDOWN)
        self.butlist.append(but)
        
        but = Button('Play movie', (butXpos[1], y), padding=6)
        but.connect_callback(self._on_play_movie_but_cbf, MOUSEBUTTONDOWN)
        self.butlist.append(but)
        
        but = Button('Pause movie', (butXpos[2], y), padding=6)
        but.connect_callback(self._on_pause_movie_but_cbf, MOUSEBUTTONDOWN)
        self.butlist.append(but)
        
        if not hidebuttons:
            self.show_buttons()

    def _on_quit_but_cbf(self, *args):
        self.logger.debug('on_quit_but_cbf called')
        self._stop()
        self._surf.fill(BLACK)
        self.observer('stop')
        
    def _on_play_movie_but_cbf(self, *args):
        self.logger.debug('_on_play_movie_but_cbf called')
        if not self._movie.get_busy():
            self._pause()
            
    def _on_pause_movie_but_cbf(self, *args):
        self.logger.debug('_on_pause_movie_but_cbf called')
        self._pause()
    
    def _stop(self):
        if self._movie.get_busy():
            self._movie.stop()
        pygame.mixer.init()
    
    def _pause(self):
        self._movie.pause()
    
    def start(self):
        pygame.mixer.quit()
        self._movie.set_volume(1.0)
        self._movie.play()
        
    def stop(self):
        self._on_quit_but_cbf()
    
    def hide_buttons(self):
        self.actives.remove(self.butlist)
        for s in self.butlist:
            s.erase_sprite()
            
    def show_buttons(self):
        self.actives.add(self.butlist)
        for s in self.butlist:
            s.display_sprite()
    
    def get_movie_surface(self):
        return self._surf

if __name__ == '__main__':
    
    import __builtin__
    __builtin__.__dict__['_'] = lambda x:x
    
    import SPLogging
    SPLogging.set_level('debug')
    SPLogging.start()
    
    from SPWidgets import Init
    
    import pygame
    from pygame.constants import *
    pygame.init() 
    
    from SPSpriteUtils import SPInit
 
    scr = pygame.display.set_mode((800, 600))
    back = scr.convert()
    actives = SPInit(scr, back)
    
    Init('seniorplay')
    
    def observer(state):
        global runloop
        print "observer called with state", state
        if state == 'stop':
            runloop = 0
    
    ####### movieplayer stuff, the rest is crap to mimic a CP environment ##########
    mprect = pygame.Rect(0, 80, 800, 500)
    mppos = mprect.topleft
    hidebuttons = False
    mp = MoviePlayer('test1.mpg', actives, mprect, observer, hidebuttons)
    mp.start()
    msurf = mp.get_movie_surface()
    ###########################
    
    runloop = 1
    clock = pygame.time.Clock()
    while runloop:
        clock.tick(30)
        pygame.event.pump()
        events = pygame.event.get()
        for event in events:
            if event.type is KEYDOWN:
                if event.key == K_ESCAPE:
                    print "escape hit, stopping loop"
                    runloop = 0
            actives.update(event)
        r = scr.blit(msurf, mppos)
        pygame.display.update(r)
        
    
    



# -*- coding: utf-8 -*-

# Copyright (c) 2007-2010 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           SPHelpText.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# This module contains classes that provide larger help text for different parts
# of schoolsplay, including the GUI.
# The objects are arranged in a way that resembles a python package
# For example, to get the text for the GUI SPCheckPass Login frame:
# ht = SPHelpText.SPCheckPass.Login

from SPConstants import HOMEDIR

#class SPCheckPass(object):
#    Login = _(\
#"If you start the GUI for the first time there's no\
# username and password set so you should just hit the 'Login' button.\n\
# You will get a warning message telling you to set as soon as possible\
# a username and password.\nYou can set a username and password in the GUI.\n\
# Go to 'Settings' -> 'Set password'.")
#     
#    TopLevel = _(\
#"Failed to get the dbase data, try to run childsplay_sp first to setup the dbase")

#class SPgdm(object):
#    _info_button_callback = _(\
#"Give your login name in the box and hit the login button.\
# Alternatively you can just hit the login button and run Childsplay_sp in\
# anonymous mode.")

class DataManager(object):
    _replace_dbase = _(\
"There was an error in the database this can be caused by many things\
 but most of the time it's caused by the fact that your version of the database\
 doesn't match the version of the program.\
 I will backup your original database and then replace it with a new one.\
 The original database is now called sp.db_back and it's placed in %s" % HOMEDIR)

    default_group_1 = _("Default group 1")
    default_group_2 = _("Default group 2")
    #default_user_1 = _("Default user 1")
    
class Nosound(object):
    ActivityStart = _(\
"It seems that we cannot use the sound card right now and because this activity\
 is all about sound, we just quit.\
 This problem can also be caused by another application which uses the\
 soundcard right now.")

class Act_ichanger(object):
    box1 = _(\
"""Memorize the pictures.

One of these pictures will be replaced by a new picture.

Touch the 'Start' button to begin the game.""")
    box2 = _(\
"""

Touch the changed picture.""")

    box3 = _(\
"""

Touch the 'Play' button to continue the game.""")
    
#class AdminGui(object):
#    on_button_help = _(\
#"This database table holds the values used when displaying the results graph in an activity.\n\
#The column 'MU-value' holds an integer between 0 and 10 indicating the average score.(the dark blue line).\n\
#The column 'SIGMA-value' holds an integer between 0 an 100 indicating the standard\
# deviation in percentage, probably always 50. (the light blue box)\n\n\
#Click on the row you want to alter and then click on the field you wish to edit.\n\
#You cannot edit the 'Activity name' field.\n\
#Use the 'Save' button to store the changes in the database.")


#class ToolTipText(object):
#    """The names of the attributes must be the same as the activity module names.
#    String must conform to the following syntax:
#    <Proper english name> <comma> <the word 'difficulty'><semi colon><space><number of 'stars' eg '*'
#    """
#    memory = _("Memory, difficulty: */**")
#    electro_sp = _("Association, difficulty: **/***")
#    findit_sp = _("Spot the differences, difficulty:*/**")
#    simon_sp = _("Repeat sound sequence, difficulty:*/***")
#
#    def get_tooltip(self, name):
#        """Returns the tooltip string or when it can't be found the activity name
#        is returnt."""
#        try:
#            attr = getattr(self, name)
#        except AttributeError:
#            s = name
#        else:
#            s = attr
#        return s

class ActivityMenuText(object):
    """These strings are used by the SPMenu module to set additional text for 
    the menu icons. This is only used when the xml attribute text is set to True
    in the menu xml. The attributes must be the name of the module."""
    usermanagement = _("Usermanagement")
    groupmanagement = _("Groupmanagement")
    dt_sequence_manual = _("Daily training sequence")
    picimport = _("Picture import")
    change_pass = _("Change password")
    change_pass_admin = _("Change password")
    update_btp = _("Update BrainTrainerPlus")
    gamemanagement = _("Gamemanagement")
    zorgenquete = "Zorg enquette"
    language_select = _("Select Language")
    settings_op = _("Settings")
    settings_adm = _("Settings")
    content_import = _("Content import")
    

class ActivityType(object):
    """These type strings are placed here so that the gettext can pick them
    up and place them in the pofile
    They are not used in the program. Perhaps in the future."""
    MemoryType = _("Memory")
    MathType = _("Math")
    PuzzleType = _("Puzzle")
    KeyboardtrainingType = _("Keyboardtraining")
    MousetrainingType = _("Mousetraining")
    LanguageType = _("Language")
    AlphabetType = _("Alphabet")
    FunType = _("Fun")
    MiscellaneousType = _("Miscellaneous")
    
    
    
    
    
